#ifndef ASTRO_H
#define ASTRO_H

#include "config.h"

#include <string.h>

/*****************************************************************************/
/*	DEFINITION DES TYPES                                                 */
/*****************************************************************************/
/* structure and definitions */
#define DEG_TO_RAD M_PI/180.0
#define SEC_TO_RAD DEG_TO_RAD/3600
#define RAD_TO_DEG 180.0/M_PI


typedef struct
{
  double amdx[43];
  double amdy[43];
  double ppo[6];
  int reduction_order;
  int pl_type;
  double xpixel_offset;
  double ypixel_offset;
  double xpix_size;
  double ypix_size;
  double ra_deg;
  double dec_deg;
} amd_wcs;


typedef struct
{
  double crval1;
  double crval2;
  double crpix1;
  double crpix2;
  double cd1_1;
  double cd1_2;
  double cd2_1;
  double cd2_2;
  double pv1[31];
  double pv2[31];
  int reduction_order;
} pv_wcs;

double distance( double ra1, double dec1, double ra2, double dec2);
void ksieta_to_radec(double ksi, double eta, double crval1, double crval2,
  double* ra, double* dec);
void cd_xypix_to_radec( double xpix, double ypix, double crval1, double crval2,
  double crpix1, double crpix2, double cd1_1, double cd1_2, double cd2_1,
  double cd2_2, double* ra_deg, double* dec_deg);
void cd_radec_to_xypix( double crval1, double crval2, double crpix1,
  double crpix2, double cd1_1, double cd1_2, double cd2_1,
  double cd2_2, double ra_deg, double dec_deg, double* xpix,
  double* ypix);
void amd_xypix_to_radec( double xpix, double ypix, amd_wcs* wcs,
  double* ra_deg, double* dec_deg);
void pv_xypix_to_radec( double xpix, double ypix, pv_wcs* wcs,
  double* ra_deg, double* dec_deg);
amd_wcs* amd_wcs_load(const char* filename);
void amd_wcs_free(amd_wcs** wcs);
pv_wcs* pv_wcs_load(const char* filename);
void pv_wcs_free(pv_wcs** wcs);

#endif
