#include <astro.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mama_utils.h>

typedef struct {
  double crval2;
  double crval1;
  float* xpix;
  float* ypix;
  int size;
  int index;
  double* ra;
  double* dec;
}interm;

static interm* interm_load(const char* filename)
{
  FILE* file = NULL;
  interm* result = NULL;
  char record[200];
  char* key = "";
  int i;
  int num;
  char sign[2];
  int rah,ram,decd,decm;
  float x,ras, decs, xmama;
  int nypavmax;
  
  MAMA_NOT_NULL( filename);
  MAMA_FOPEN( file, filename, "r");
  MAMA_CALLOC( result, interm, 1);

  while(fgets(record,200,file) && (strstr(key, "END") == NULL) ){
    key = strtok(record, "=");
    char* value = strtok(NULL,"/\n");

    if ( strstr(key, "CRVAL2") != NULL){
      result->crval2 = atof(value);
    }
    else if ( strstr(key, "CRVAL1") != NULL){
      result->crval1 = atof(value);
    }
    else if ( strstr(key, "NYPAVMAX") != NULL){
      nypavmax = atoi(value);
    }
  }
  while( sscanf(record, "%d", &num) != 1){
    fgets(record,200,file);
  }
  /* allocate many objects */ 
  result->size = 8000;
  MAMA_CALLOC(result->xpix, float, result->size);
  MAMA_CALLOC(result->ypix, float, result->size);
  MAMA_CALLOC(result->ra, double, result->size);
  MAMA_CALLOC(result->dec, double, result->size);
  result->index = 0;

  /* get data */
  sscanf(record,"%*d %*d %f %*f %d %d %f %1s %d %d %f %*f %*f %*d %*f %*f %f",&xmama, &rah,&ram,&ras,sign,&decd,&decm,&decs,&(result->xpix[result->index]));
    result->ra[result->index] = (rah+ram/60.0+ras/3600.0)*15;
    result->dec[result->index] = decd+decm/60.0+decs/3600.0;
    if (strstr(sign,"-") != NULL){
      result->dec[result->index] = -(result->dec[result->index]);
    }
    result->ypix[result->index] = nypavmax*1024.0+0.5-(xmama/1.000033-result->crval2)*100;

    while(fgets(record,200,file)){
      int nb = 0;
    
      nb = sscanf(record,"%*d %*d %f %*f %d %d %f %1s %d %d %f %*f %*f %*d %*f %*f %f", &xmama, &rah, &ram, &ras, sign,
        &decd,&decm,&decs,&x);
      if (nb == 9) {
	result->index++;
        result->xpix[result->index] =x;
        result->ra[result->index] = (rah+ram/60.0+ras/3600.0)*15;
        result->dec[result->index] = decd+decm/60.0+decs/3600.0;
	if (strstr(sign,"-") != NULL){
	  result->dec[result->index] = -(result->dec[result->index]);
	}
        result->ypix[result->index] = nypavmax*1024.0+0.5-(xmama/1.000033-result->crval2)*100;
      }
    } 
  fclose( file);
  return result;
}


int main(int argc, char* argv[])
{
  double pvra = 0.0, pvdec =0.0, amdra = 0.0, amddec = 0.0;
  double pvramean, pvdecmean, pvrasigma, pvdecsigma, pvdiffra, pvdiffdec;
  double amdramean, amddecmean, amdrasigma, amddecsigma, amddiffra, amddiffdec;
  int i = 0.0;
  
  interm* in = NULL;
  pv_wcs* pvwcs = NULL;
  amd_wcs* amdwcs = NULL;
  
  pvwcs = pv_wcs_load(argv[1]);
  amdwcs = amd_wcs_load(argv[2]);
  in = interm_load(argv[3]);
  
  pvramean = pvdecmean = pvrasigma = pvdecsigma = 0.0;
  amdramean = amddecmean = amdrasigma = amddecsigma = 0.0;

  for(i=0; i< in->index; i++){
    pv_xypix_to_radec( in->xpix[i], in->ypix[i], pvwcs, &pvra, &pvdec);
    pvdiffra = (in->ra[i]-pvra)*3600.0*cos(pvdec*DEG_TO_RAD);
    pvdiffdec = (in->dec[i]-pvdec)*3600.0;
    amd_xypix_to_radec( in->xpix[i], in->ypix[i], amdwcs, &amdra, &amddec); 
    amddiffra = (in->ra[i]-amdra)*3600.0*cos(amddec*DEG_TO_RAD);
    amddiffdec = (in->dec[i]-amddec)*3600.0;
    printf("x %.10E y %.10E ucacra %.10E ucacdec %.10E pvra %.10E pvdec %.10E amdra %.10E amddec %.10E \
	    pvdiffra %.10E pvdiffdec %.10E amddiffra %.10E amddiffdec %.10E\n",in->xpix[i],
      in->ypix[i], in->ra[i], in->dec[i], pvra, pvdec, amdra, amddec, pvdiffra, pvdiffdec, amddiffra, amddiffdec);
    pvramean += pvdiffra;
    pvdecmean += pvdiffdec;
    pvrasigma += pvdiffra*pvdiffra;
    pvdecsigma +=  pvdiffdec*pvdiffdec;
    
    amdramean += amddiffra;
    amddecmean += amddiffdec;
    amdrasigma += amddiffra*amddiffra;
    amddecsigma +=  amddiffdec*amddiffdec;
  }
  i = i+1;
  pvramean /=  i;
  pvdecmean /= i;
  pvrasigma /= i;
  pvdecsigma /= i;
  pvrasigma = sqrt(pvrasigma);
  pvdecsigma = sqrt(pvdecsigma);
  
  amdramean /=  i;
  amddecmean /= i;
  amdrasigma /= i;
  amddecsigma /= i;
  amdrasigma = sqrt(amdrasigma);
  amddecsigma = sqrt(amddecsigma);
  
  printf("PV : nbiter %d ramean %.10E decmean %.10E rasigma %.10E decsigma %.10E\n",i, pvramean,pvdecmean,pvrasigma,pvdecsigma);
  printf("AMD : nbiter %d ramean %.10E decmean %.10E rasigma %.10E decsigma %.10E\n",i, amdramean,amddecmean,amdrasigma,amddecsigma);
  return 0;
}
