#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>
#include <fits.h>
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* SYNTAX definitions
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

#define        CUTBORDER_SYNTAX \
"cutBorder     version " PACKAGE_VERSION "\n"\
"              extract image borders\n\
             -file         <image to cut>\n\
             -top          <size to cut 0=nocut>\n\
             -bottom       <size to cut 0=nocut>\n\
             -left         <size to cut 0=nocut>\n\
             -right        <size to cut 0=nocut>\n\
             -dir          <directory to write the result>\n\
             OPTIONNAL parameters\n\
             -left-space   <left origin cut>\n\
             -right-space  <right origin cut>\n\
             -top-space    <top origin cut>\n\
             -bottom-space <bottom origin cut>\n"

#define XBANDE_SIZE 1536  /* in pixels */
#define YBANDE_SIZE 1024

#define OBJECT_SIZE 10000

#define LEVEL 128

#define PLATE_BORDER_LEVEL 0
#define BACKGROUND_LEVEL 2

void cut_8bit(char* source,char* result,int xdep, int ydep, int naxis1, int naxis2){
  fits* f = NULL, *fcut = NULL;
  char value[22];
  signed short* buff16 = NULL;
  char* buff8 = NULL;
  FILE* file = NULL;
  int nb,i,j;

  f = fits_new(source);
  /* verify the parameters */
  if(f->bitpix != 16){
    mama_error(-1,"Can't convert this file bitpix != 16");
  }

  fcut = fits_template_new(f,result);

  fits_set_bitpix(fcut,8);
  fits_set_naxis1(fcut,naxis1);

  fits_set_naxis2(fcut,naxis2);

  sprintf(value,"%20f", 0.0);
  fits_keyword_set(fcut,"BZERO",value);
  sprintf(value,"%20f", 1.0);
  fits_keyword_set(fcut,"BSCALE",value);
  sprintf(value,"%20d",xdep);
  fits_keyword_add(fcut,"XDEP",value);
  sprintf(value,"%20d",ydep);
  fits_keyword_add(fcut,"YDEP",value);

  /* go to the pixel */
  fits_seek_pixels(f);

  /* write the result file */
  file = fopen(result,"w");
  if( file == NULL ){
    mama_error(-1,"Can't write in result file");
  }
  nb = fwrite(fcut->header, 2880 * fcut->nb_header_block,1,file);

  if (nb != 1) {
    mama_error(-1,"Can't write the header");
  }

  /* write the pixel */
  buff8 = (char*)(malloc(fcut->naxis1*sizeof(char)));
  buff16 = (signed short*)(malloc(f->naxis1*sizeof(signed short)));

  fseek(f->file,ydep*f->naxis1*2, SEEK_CUR);

  for(j=0;j< naxis2;j++){
    nb = fread(buff16,f->naxis1,sizeof(signed short),f->file);
    swap16( buff16, f->naxis1);
    for(i=0;i<naxis1;i++){
      buff8[i]=(char)(rint((buff16[xdep+i]+32768.0)/64535.0*(LEVEL-1)));
    }
    nb = fwrite(buff8,naxis1,sizeof(char),file);
  }
  /* complete with zero */
  fits_complete_with_zero(fcut,file);

  fclose(file);

  fits_free(fcut);
  free(buff8);
  free(buff16);
  fits_free(f);
}


#define X_WIN_SIZE 30
#define Y_WIN_SIZE 30

static char avg(fits* f, char* data, int ai, int aj, int bi, int bj)
{
  long avg;
  int nb,i,j;

  nb = 0;
  for(j=aj; j<=bj; j++){
    for(i=ai; i<= bi; i++){
      avg += data[i+f->naxis1*j];
      nb++;
    }
  }
  avg = avg/nb;

  return (char)avg;
}


static void filter_avg(fits* f, char** data)
{
  int i,j;
  char* res = NULL;

  res = (char*)(calloc(f->naxis1*f->naxis2,sizeof(char)));

  for(j=0;j< f->naxis2;j++){
      for(i=0; i<f->naxis1;i++){
          int ai,aj,bi,bj;

          ai = i-X_WIN_SIZE;
          aj = j-Y_WIN_SIZE;
          bi = i+X_WIN_SIZE;
          bj = j+Y_WIN_SIZE;

          if ( (i-X_WIN_SIZE) < 0){
            ai = 0;
          }
          if ( (i+X_WIN_SIZE) > (f->naxis1-1)){
            bi = f->naxis1-1;
          }
          if ( (j-Y_WIN_SIZE) < 0){
            aj = 0;
          }
          if ( (j+Y_WIN_SIZE) > (f->naxis2-1)){
            bj = f->naxis2-1;
          }
          res[i+j*f->naxis1] = avg(f,*data,ai,aj,bi,bj);
        }
      }
  free(*data);
  *data = res;
}

static void filter_avg_test(fits* f, char** data,long* sumx,long avgx, long* sumy , long avgy)
{
  int i,j;
  char* res = NULL;

  res = (char*)(calloc(f->naxis1*f->naxis2,sizeof(char)));

  for(j=0;j< f->naxis2;j++){
    if(sumx[j]>= avgx){
      for(i=0; i<f->naxis1;i++){
        if (sumy[i]>=avgy){
          int ai,aj,bi,bj;

          ai = i-X_WIN_SIZE;
          aj = j-Y_WIN_SIZE;
          bi = i+X_WIN_SIZE;
          bj = j+Y_WIN_SIZE;

          if ( (i-X_WIN_SIZE) < 0){
            ai = 0;
          }
          if ( (i+X_WIN_SIZE) > (f->naxis1-1)){
            bi = f->naxis1-1;
          }
          if ( (j-Y_WIN_SIZE) < 0){
            aj = 0;
          }
          if ( (j+Y_WIN_SIZE) > (f->naxis2-1)){
            bj = f->naxis2-1;
          }
          res[i+j*f->naxis1] = avg(f,*data,ai,aj,bi,bj);
        }
else{
  res[i+j*f->naxis1] = (*data)[i+j*f->naxis1];
}
      }
    }
    else{
      for(i=0; i<f->naxis1;i++){
        res[i+j*f->naxis1] = (*data)[i+j*f->naxis1];
      }
    }
  }
  free(*data);
  *data = res;
}

void filter_background(fits* f, char* data){
  long* levels = NULL;
  long max = 0;
  char level = 0;
  char maxlevel = 0;
  char filter_level = 0;
  int i,j;

  printf("level %d\n",LEVEL);
  levels = (long*)(calloc(LEVEL,sizeof(long)));

/*  for(i=0; i< LEVEL; i++){
    printf("level %d %d\n",i,levels[i]);
  }*/
  for(j=0;j< f->naxis2;j++){
    for(i=0; i<f->naxis1;i++){
      int a;
      a = (int)data[j*f->naxis1+i];
      levels[a] = levels[a]+1;
    }
  }
/*  for(i=0; i< LEVEL; i++){
    printf("level %d %d\n",i,levels[i]);
  }
*/
  /* find the median of level */
  for(i=1;i<LEVEL;i++){
    if(levels[i] > max){
      max = levels[i];
      level = (char)i;
    }
  }
  printf("background level %d\n",level);
  /* get the value on max/2 */
  filter_level = level;
  while( filter_level < (LEVEL-1) && (levels[filter_level] > max/2)){
    filter_level++;
  }

  maxlevel = (char)(LEVEL * 0.8); 
  /* filter the data with subtract background */
  for(j=0;j< f->naxis2;j++){
    for(i=0;i<f->naxis1;i++){
      if(data[j*f->naxis1+i] <= filter_level && data[j*f->naxis1+i]>0){
        data[j*f->naxis1+i] = BACKGROUND_LEVEL;
      }
      else if(data[j*f->naxis1+i] >= maxlevel || (data[j*f->naxis1+i] == 0)){
        data[j*f->naxis1+i] = LEVEL-1;
      }
    }
  }

  free(levels);
}

void filter_invert(fits* f,char* data){
  int i,j;

  /* filter the data with subtract background */
  for(j=0;j< f->naxis2;j++){
    for(i=0;i<f->naxis1;i++){
      if(data[j*f->naxis1+i] == 1){
        data[j*f->naxis1+i] = 0;
      }
      if(data[j*f->naxis1+i] == (LEVEL-1)){
        data[j*f->naxis1+i] = 1;
      }
   }
 }
}

void contour_image(char* file, char* result,int mode){
  fits* f = NULL;
  char* data = NULL;
  int nb,i,j;
  FILE* image = NULL;

  f = fits_new(file);

  fits_seek_pixels(f);

  data = (char*)(malloc(f->naxis1*f->naxis2*sizeof(char)));
  nb = fread(data,f->naxis1*f->naxis2,sizeof(char),f->file);

  /*filter_background(f,data);
  filter_small_objects(f,data);
  filter_invert(f,data);
  */
  filter_background(f,data);

  if( mode > 0) {
    long* sumx, *sumy;
    long avgx,avgy;
    sumx = (long*)calloc(f->naxis2,sizeof(long));
    sumy = (long*)calloc(f->naxis1,sizeof(long));
    for(j=0;j<f->naxis2;j++){
      for(i=0; i< f->naxis1; i++){
        sumx[j]+=data[i+j*f->naxis1];
sumy[i]+=data[i+j*f->naxis1];
      }
    }
    avgx = 0;
    avgy = 0;
    for(i=0; i< f->naxis2; i++){
      avgx += sumx[i];
    }
    
    for(i=0; i< f->naxis1; i++){
      avgy += sumy[i];
    }
    avgx /=f->naxis2;
    avgy = 0;
    printf ("avgx %ld avgy %ld\n",avgx,avgy);
    filter_avg_test(f,&data,sumx,avgx,sumy,avgy);
//    filter_avg(f,&data);
  }
  /* write the result */
  image = fopen(result,"w");
  if( image == NULL ){
    mama_error(-1,"Can't write in result file");
  }
  nb = fwrite(f->header, 2880 * f->nb_header_block,1,image);
  nb = fwrite(data,f->naxis1*f->naxis2,sizeof(char),image);
  fits_complete_with_zero(f,image);

  fclose(image);
  free(data);
  fits_free(f);
}



int main(int argc,char* argv[]){
  char* filename = NULL;
  char* dirname = NULL;
  char name[1024];
  char * root = NULL;
  fits* source = NULL;
  int left_space = 0, top_space = 0, bottom_space = 0, right_space = 0;
  int left_ox, left_oy, right_ox, right_oy, top_ox, top_oy, 
      bottom_ox, bottom_oy;
  int i, top = 0, bottom = 0, left = 0, right = 0;

  if(argc < 5){
    printf("%s",CUTBORDER_SYNTAX);
    exit(-1);
  }
  for( i=1;i<argc;i++ ){
    if( strcmp("-file",argv[i])==0 ){
      filename = strdup(argv[++i]);
      filename[strlen(argv[i])]='\0';
    }
    else if ( strcmp("-dir",argv[i])==0 ){
      dirname = strdup(argv[++i]);
    }
    else if ( strcmp("-top",argv[i])==0 ){
      top = atoi(argv[++i]);
    }
    else if ( strcmp("-bottom",argv[i])==0 ){
      bottom = atoi(argv[++i]);
    }
    else if ( strcmp("-left",argv[i])==0 ){
      left = atoi(argv[++i]);
    }
    else if ( strcmp("-right",argv[i])==0 ){
      right = atoi(argv[++i]);
    } 
    else if ( strcmp("-top-space",argv[i])==0 ){
      top_space = atoi(argv[++i]);
    }
    else if ( strcmp("-bottom-space",argv[i])==0 ){
      bottom_space = atoi(argv[++i]);
    }
    else if ( strcmp("-left-space",argv[i])==0 ){
      left_space = atoi(argv[++i]);
    }
    else if ( strcmp("-right-space",argv[i])==0 ){
      right_space = atoi(argv[++i]);
    }
    else {
      printf("%s",CUTBORDER_SYNTAX);
      exit(-1);
   }
 }
  
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("CUTBORDER\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("Source file  = %s\n", filename);
  printf("Directory  = %s\n", dirname);

  source = fits_new(filename);
  left_ox = left_space;
  left_oy = 0;
  bottom_ox = 0;
  bottom_oy = bottom_space;
  right_ox = source->naxis1-1-right-right_space; 
  right_oy = 0;
  top_ox = 0;
  top_oy = source->naxis2-1-top-top_space;

  printf("LEFT   : ORIG (%d,%d) SIZE %d\n",left_ox,left_oy,left);
  printf("BOTTOM : ORIG (%d,%d) SIZE %d\n",bottom_ox,bottom_oy,bottom);
  printf("RIGHT  : ORIG (%d,%d) SIZE %d\n",right_ox,right_oy,right);
  printf("TOP  : ORIG (%d,%d) SIZE %d\n",top_ox,top_oy,top);

  root = strdup(strrchr(filename,'/'));
  root = strtok(root,".");

  printf("Going to cut\n");
  /* create the cut */
  if( bottom > 0 ){
    sprintf(name,"%s/%s_bottom.fits",dirname,root); 
    cut_8bit(filename,name,bottom_ox, bottom_oy,source->naxis1, bottom);
  }
  if ( top > 0 ){
    sprintf(name,"%s/%s_top.fits",dirname, root); 
    cut_8bit(filename, name, top_ox, top_oy, source->naxis1, top); 
  }
  if ( left > 0){
    sprintf(name,"%s/%s_left.fits",dirname, root); 
    cut_8bit(filename,name, left_ox, left_oy, left, source->naxis2); 
  }
  if ( right > 0){
    sprintf(name,"%s/%s_right.fits",dirname, root); 
    cut_8bit(filename,name, right_ox, right_oy, right, source->naxis2); 
  }
/* create the contour */
/*
  sprintf(name,"%s/bottom.fits",dirname);
  sprintf(contourname,"%s/bottom_contours.fits",dirname);
  contour_image(name,contourname,0);
  sprintf(name,"%s/top.fits",dirname);
  sprintf(contourname,"%s/top_contours.fits",dirname);
  contour_image(name,contourname,0);
  sprintf(name,"%s/left.fits",dirname);
  sprintf(contourname,"%s/left_contours.fits",dirname);
  contour_image(name,contourname,1);
  sprintf(name,"%s/right.fits",dirname);
  sprintf(contourname,"%s/right_contours.fits",dirname);
  contour_image(name,contourname,1);
*/
  fits_free(source);
  free(filename);
  free(dirname);
  return 0;

}



