#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>

#include "fits.h"


/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* fits_new
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
fits* fits_new(char* filename){
  fits* new = NULL;
  int end =0;
  int nread = 0;
  char value[80];
 
  new = (fits*)(malloc(sizeof(fits)));
  if( new == NULL ){
    printf("Can't create fits structure\n");
    exit(-1);
  }

  new->filename = strdup(filename);
  new->file = fopen(filename, "r");

  if( new->file == NULL ){
    printf("Can't read fits file %s\n",filename);
    exit(-1);
  }
  /* create fits header */
  new->nb_header_block = 1;
  
  /* find the number of keywords */
  new->nbKeys = 0;

  while(!end){
    char line[80];
    
    nread = fread(line,80, 1, new->file);
    if( nread!=1){
      printf("Can't read header of fits file\n");
      exit(-1);
    }
    end = (strncmp(line,"END",3)==0);
    new->nbKeys++;
  }
  new->nb_header_block = ceil((double)new->nbKeys/36.0);
  
  new->header = (char*)(malloc(new->nb_header_block*2880*sizeof(char)));
  if( new->header == NULL ){
    mama_error(-1,"Not enough memory for header of fits file");
  }
  rewind(new->file);
  nread = fread(new->header,2880*new->nb_header_block, 1, new->file);
  if( nread!=1){
      mama_error(-1,"Can't read fits file");
  }

  if(fits_keyword_get(new,"NAXIS1",value)){
    new->naxis1 = atol(value);
  }
  else {
    mama_error(-1,"Can't read NAXIS1 in header");
  }

  if(fits_keyword_get(new,"NAXIS2",value)){
    new->naxis2 = atol(value);
  }
  else {
    mama_error(-1,"Can't read naxis2 in header");
  }
  
  if(fits_keyword_get(new,"BITPIX",value)){
    new->bitpix = atoi(value);
    if( new->bitpix == 16 ){
      new->bitpix_size = sizeof(short);
    }
    else if( (new->bitpix == -32)||(new->bitpix == 32)){
      new->bitpix_size = sizeof(float);
    }
    else if( new->bitpix == 8 ){
          new->bitpix_size = sizeof(char);
    }
    else{
      printf("Unknown bitpix %d\n",new->bitpix);
      exit(-1);
    }
  }
  else {
    mama_error(-1,"Can't read bitpix in header");
  }
  return new;
}

/*
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * * fits_template_new
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * */
fits* fits_template_new(fits* file, char* filename){
  fits* new = NULL;

  new = (fits*)(malloc(sizeof(fits)));
  new->filename  = strdup(filename);
  
  new->nb_header_block = file->nb_header_block;
  new->nbKeys = file->nbKeys; 
  new->naxis1 = file->naxis1;
  new->naxis2 = file->naxis2;
  new->bitpix = file->bitpix;
  new->bitpix_size = file->bitpix_size;
  new->file = NULL; 
  new->header = (char*)(malloc(new->nb_header_block*2880*sizeof(char)));

  strncpy(new->header,file->header,new->nb_header_block*2880*sizeof(char));

  return new;
}

/*
 * ----------------------------------------------------------------------------------------
 *
 */

/*
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * * fits_set_naxis1
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void fits_set_naxis1(fits* file,long naxis1){
  if(file->naxis1 != naxis1){
    file->naxis1 = naxis1;
    fits_keyword_set_long(file,"NAXIS1",naxis1);
  }
}

/*
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * * fits_set_naxis2
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void fits_set_naxis2(fits* file,long naxis2){
  if(file->naxis2 != naxis2){
    file->naxis2 = naxis2;
    fits_keyword_set_long(file,"NAXIS2",naxis2);
  }
}


/*
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * * fits_set_bitpix
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void fits_set_bitpix(fits* file,int bitpix){
  if(file->bitpix != bitpix){
    if( bitpix == 16 ){
      file->bitpix_size = sizeof(short);
    }
    else if( (bitpix == -32)||(bitpix == 32)){
      file->bitpix_size = sizeof(float);
    }
    else if( bitpix == 8 ){
      file->bitpix_size = sizeof(char);
    }
    else{
      printf("Unknown bitpix %d\n",file->bitpix);
      exit(-1);
    }
    fits_keyword_set_long(file,"BITPIX",bitpix);
  }
}

/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* fits_seek_pixels
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void fits_seek_pixels(fits* file){
  fseek(file->file,file->nb_header_block*2880,SEEK_SET);
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* fits_seek_pixels_end
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void fits_seek_pixels_end(fits* file){
  fseek(file->file,file->nb_header_block*2880+file->naxis1*file->naxis2*
    file->bitpix_size,SEEK_SET);
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* fits_complete_with_zero
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void fits_complete_with_zero(fits* file,FILE* result){
  long int image_size = 0;
  long int zero_buffer_size = 0;
  long int nb_data_blocks = 0;
  char* zero_buffer = NULL;
  int nbwrite = 0;  

  /* compute pixel size */
  image_size = file->naxis1*file->naxis2*file->bitpix_size;
  
  /*compute zero buffer size */
  nb_data_blocks = ceil((double)image_size / 2880.0);
  zero_buffer_size = nb_data_blocks*2880-image_size;
  zero_buffer = (char*)(calloc(zero_buffer_size,sizeof(char)));
  nbwrite = fwrite(zero_buffer,zero_buffer_size, 1,result);
  /* free memory */
  free(zero_buffer);
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* fits_free
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void fits_free(fits* file){
  if(file){
    if(file->filename)free(file->filename);
    if (file->header) free(file->header);
    if (file->file) fclose(file->file);
    free(file);
  }
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* fits_keyword_get
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int fits_keyword_get(const fits* file, char* keyword, char* value){
  int i;
  int res = 0;

  for(i=0;i<file->nbKeys;i++){    
    char* line;
    line = &file->header[80*i];
    if( keyword_get(keyword,value,line) ){
      res = 1;
      break;
    }
  }
  return res;
}
/*
----------------------------------------------------------------------------------------
*/


int fits_keyword_set_long(fits* file,char* keyword,long value){
  char values[22];

  sprintf(values,"%20ld",value);
  return fits_keyword_set(file,keyword,values);
}


/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* fits_keyword_set
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int fits_keyword_set(fits* file,char* keyword,char* value){
  int i,j,k;
  char oldvalue[72];
  

  for(i=0;i<file->nbKeys;i++){
    char* line;
    
    line = &file->header[80*i];
    if( keyword_get(keyword,oldvalue,line) ){
      int size;

      size = strlen(value);

      if(size > 70){
        size = 70;
      }

      for (j = 0; j < size; j++){
        /* suppress the comment if exists */
	if( line[10+ j]=='/' ){
	  for(k=j;k<size;k++){
	    line[10+k] = ' ';
	  }
	}
        /* write the value */
        line[10+j] = value[j];
      }
      for (j = 10+size; j < 80; j++){
        line[j] = ' ';
      }
      return 1;
    }
  }
  return 0;
}
/*
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * * fits_keyword_rename
 * ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 * */
int fits_keyword_rename(fits* file,char* keyword,char* value){
  int i,j;
  char oldvalue[72];
  char newKey[8];

  for(i=0;i<file->nbKeys;i++){
    char* line;
 
    line = &file->header[80*i];
    if( keyword_get(keyword,oldvalue,line) ){
      sprintf(newKey,"%-8s",value);
      for(j=0; j< 8; j++){
        line[j] = newKey[j];
      }
      return 1;
    }
  }
  return 0;
}


/*
----------------------------------------------------------------------------------------
*/


/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* fits_keyword_add
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int fits_keyword_add(fits* file,char* keyword,char* value){

  if(fits_keyword_set(file,keyword,value) == 0){
    /* add the keyword */
    char* line;
    line = &file->header[80*(file->nbKeys-1)];
    sprintf(line,"%-8s= %-70s",keyword,value);
    line = &file->header[80*file->nbKeys];
    sprintf(line,"%-80s","END");
    file->header[80*(file->nbKeys+1)] = ' ';
    file->nbKeys = file->nbKeys + 1;
    return 1;
  }
  return 0;
}
/*
----------------------------------------------------------------------------------------
*/

/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* keyword_get
* X+Warning value must have size of 70
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int keyword_get(char* key, char* value, const char* line){
  int i = 0,len = 0;
  char keyword[9];
  
  len = strlen(key);
  strncpy(keyword,key,8);
  
  for(i=len;i<8;i++){
	  keyword[i]=' ';
  }
  keyword[8] = '\0';
  if( strncmp( line, keyword, 8)==0 ){
    i = 10;
    while( (i<80) && (line[i] != '/') ){
      value[i-10] = line[i];
      i++;
    }
    value[i-10]='\0';
    return 1;
  }
  else{
    return 0;
  }
}
/*
----------------------------------------------------------------------------------------
*/
