#ifndef FITS_H
#define FITS_H

#include "config.h"

#include <string.h>


#define mama_error(error_code, format,...)\
  printf(format,##__VA_ARGS__);\
  exit(error_code)
/*****************************************************************************/
/*	DEFINITION DES TYPES                                                 */
/*****************************************************************************/

/* PIXELS */
typedef unsigned short PIXTYPE;

/* FITS file */
typedef struct
{
	FILE *file;
	char* filename;
	char *header;
	int nbKeys;
	int nb_header_block;
	unsigned long naxis1;
	unsigned long naxis2;
	int bitpix;
        int bitpix_size;
} fits;

/******************************************************************************/
/*      PROTOTYPES                                                            */
/******************************************************************************/

fits* fits_new(char* filename);
fits* fits_template_new(fits* file,char* filename);
void fits_free(fits* file);

void fits_set_naxis1(fits* file,long naxis1);
void fits_set_naxis2(fits* file,long naxis2);
void fits_set_bitpix(fits* file,int bitpix);

int fits_keyword_get(const fits* file,char* keyword,char* value);
int fits_keyword_set_long(fits* file,char* keyword,long value);
int fits_keyword_set(fits* file,char* keyword,char* value);
int fits_keyword_rename(fits* file,char* keyword1,char* keyword2);
int fits_keyword_add(fits* file,char* keyword,char* value);
int keyword_get(char* key, char* value, const char* line);
void fits_complete_with_zero(fits* file,FILE* result);
void fits_seek_pixels_end(fits* file);
void fits_seek_pixels(fits* file);

/******************************************************************************/
/*      inline functions                                                      */
/******************************************************************************/
static inline void swap16(PIXTYPE* buffer,int size){
  if (SWAP){
    char *elements;
    int i;
    for (i = 0; i < size; i++) {
        elements = (char *)&buffer[i];
        elements[0] ^= (elements[1] ^= (elements[0] ^= elements[1]));
    }
  }
}

static inline void swap32(float* buffer,int size){
  if (SWAP){
    char* elements;
    char tmp[4];
    int i;
    for (i = 0; i < size; i++) {
      elements = (char *)&buffer[i];
      tmp[3] =  *elements;
      tmp[2] =  *(elements+1);
      tmp[1] =  *(elements+2);
      tmp[0] =  *(elements+3);
      memcpy(elements, tmp, 4 );
    }
  }
}

#endif
