#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <getopt.h>
#include <fits.h>
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* SYNTAX
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

#define         MBPMAP_SYNTAX   \
"mbmap      <construct a bpmap\n\
                -file \n\
Example :\n\
  mbmap -file mymosaique.fits -value 1"



/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* mbpmap
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int main(int argc,char* argv[]){

  int i,j,nb;
  FILE* file = NULL;
  fits* result = NULL;
  char* filename = NULL;
  char* rootname = NULL;
  char mapname[128];
  char key[22];
  unsigned short *buff1 = NULL;
  float* line = NULL;
  float bzero = 1.802240;
  float bscale = 0.000055;
   if(argc < 2){
      printf("%s",MBPMAP_SYNTAX);
     exit(-1);
   }
  for( i=1;i<argc;i++ ){
    if( strcmp("-file",argv[i])==0 ){
      filename = strdup(argv[++i]);  
    }
    else {
      printf("%s",MBPMAP_SYNTAX);
      exit(-1);
    }
  }

  if (filename == NULL){
    printf("You must provide a non null filename\n");
    exit(-1);
  }
  rootname = strtok(strdup(filename),".");
  sprintf(mapname,"%s_inv.fits",rootname);
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("mbpmap\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");

  result = fits_new(filename);
  sprintf(key,"                 -32");
  fits_keyword_set(result,"BITPIX",key);
  sprintf(key,"                   0");
  fits_keyword_set(result,"BZERO",key);
  sprintf(key,"                   1");
  fits_keyword_set(result,"BSCALE",key);
/* open the sub image file */
  file = fopen(mapname,"w");
  if( file == NULL ){
    printf("Can't write file %s\n",mapname);
    exit(-1);
  }
  /* write in the sub image file */
  nb = fwrite(result->header, 2880 * result->nb_header_block,1,file);
  if (nb!= 1) {
    printf("Can't write the header in the sub image file %s\n",mapname);
    exit(-1);
  }

  fits_seek_pixels(result);

  buff1 = (unsigned short*)(malloc(sizeof(unsigned short)*result->naxis1));
  line = (float*)(malloc(sizeof(float)*result->naxis1));
   
 for(j=0;j<result->naxis2;j++){
    nb = fread(buff1,result->naxis1,sizeof(unsigned short),result->file);
    swap16(buff1,result->naxis1);
    for(i=0;i<result->naxis1;i++){
      line[i] = -(buff1[i]*bscale+bzero);
    }
    swap32(line,result->naxis1);
    nb = fwrite(line,sizeof(float),result->naxis1,file);
  }
  free(buff1);
  free(line);
  fits_complete_with_zero(result,file);
  fclose(file);
  fits_free(result);
  free(rootname);
  free(filename);
  return 0;
}
