#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>
#include <fits.h>
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* SYNTAX definitions
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

#define        CONVERT_SYNTAX \
"mconvert      convert a mosaic image 32 bits to 16 bits >\n\
               -file   < file to convert>\n\
               -output < result file>\n"


int main(int argc,char* argv[]){
  char* filename = NULL;
  char* resultname = NULL;
  fits* result = NULL;
  float* buff32 = NULL;
  short* buff16 = NULL;
  float max;
  float bscale = 1.0, bzero =0.0;
  long i,j;
  double cte;
  int nbread;
  char value[22];
  FILE* file = NULL;
  char* zero = NULL;
  int size_to_complete;

  if(argc < 4){
    printf("%s",CONVERT_SYNTAX);
    exit(-1);
  }
  for( i=1;i<argc;i++ ){
    if( strcmp("-file",argv[i])==0 ){
      filename = strdup(argv[++i]);  
    }    
    else if(strcmp("-output",argv[i])==0 ){
      resultname = strdup(argv[++i]);       
    }
    else{
      printf("%s",CONVERT_SYNTAX);
      exit(-1);
    }
  }
  
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("MCONVERT 32 bits file to 16 bits MAMA\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("File to convert = %s\n", filename);
  printf("Result       = %s\n", resultname);
  
  result = fits_new(filename);
  /* verify the parameters */
  if(result->bitpix != -32){
    error(-1,"Can't convert this file bitpix != -32");
  }
  
  /* write new bitpix, bscale and bzero */
  bscale = 0.000055;
  bzero = 32768.0*bscale;
  sprintf(value,"%20d\0",16);
  fits_keyword_set(result,"BITPIX",value);
  sprintf(value,"%20f\0",bzero);
  fits_keyword_set(result,"BZERO",value);
  sprintf(value,"%20f\0",bscale);
  fits_keyword_set(result,"BSCALE",value);
  fits_keyword_set(result,"RADECSYS","'ICRS          '");
  /* compute the 16 bits image */
  
  /* go to the pixel */
  fits_seek_pixels(result);
  
  /* write the result file */
  file = fopen(resultname,"w");
  if( file == NULL ){ 
    error(-1,"Can't write in result file");
  } 
  /* write the header */
  nbread = fwrite(result->header, 2880 * result->nb_header_block,1,file);
  if (nbread != 1) {
    error(-1,"Can't write the header in the convert file");
  }

  /* write the pixel */
  buff16 = (short*)(malloc(result->naxis1*sizeof(short)));
  buff32 = (float*)(malloc(result->naxis1*sizeof(float)));

  for(i=0;i<result->naxis2;i++){
    nbread = fread(buff32,result->naxis1,sizeof(float),result->file);
    swap32(buff32,result->naxis1);

    for(j=0;j<result->naxis1;j++){
      if(buff32[j]<0.0){
        buff32[j] = 0.0;
      }
      buff16[j] = (short)(rint((buff32[j]-bzero)/bscale));
      
    }
    swap16((PIXTYPE*)buff16,result->naxis1);
    nbread = fwrite(buff16,result->naxis1,sizeof(short),file);
  }

  /* complete with zero */
  result->bitpix_size = 2;
  fits_complete_with_zero(result,file);
  fclose(file);
  
  free(buff32);
  free(buff16);
  fits_free(result);
  free(filename);
  free(resultname);
  return 0;
}
