#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>
#include <values.h>  /* M_PI defintion */
#include <fits.h>
#include <astro.h>

/* structure and definitions */
#define DEG_TO_RAD M_PI/180.0
#define RAD_TO_DEG 180.0/M_PI
#define DMIN 1 /* en s d'arc */

struct sexObj_ {
int number;
float mag_auto;
float mag_err_auto;
double background;
double flux_max;
int isoarea_image;
double x_image;
double y_image;
double a_image;
double b_image;
double ra;
double dec;
};

typedef struct sexObj_ sexObj;

struct ucac2Obj_{
char name[256];
double ra;
double dec;
float mag;
double raDate;
double decDate;
double epRa;
double epDec;
double pmRa;
double pmDec;
};

typedef struct ucac2Obj_ ucac2Obj;


struct kw_{
int naxis1;
int naxis2;
double crpix1;
double crpix2;
double crval1;
double crval2;
double cd1_1;
double cd2_2;
double dated;
};

typedef struct kw_ kw;


void readKW(char* imagename,kw* h){
  fits* result = NULL;
  char value[71];

  result = fits_new( imagename);

  /* read kw */
  if( fits_keyword_get(result, "NAXIS1", value)){
    h->naxis1 = atoi(value);
  }
  else {
    mama_error(-1,"Can't read NAXIS1 in header\n");
  }
  if(fits_keyword_get(result, "NAXIS2", value)){
    h->naxis2 = atoi(value);
  }
  else {
    printf("Can't read NAXIS2 in header\n");
    exit(-1);
  }
  if(fits_keyword_get(result,"CRPIX1",value)){
    h->crpix1 = atof(value);
  }
  else {
    printf("Can't read CRPIX1 in header\n");
    exit(-1);
  }
  if(fits_keyword_get(result,"CRPIX2",value)){
    h->crpix2 = atof(value);
  }
  else {
    printf("Can't read CRPIX2 in header\n");
    exit(-1);
  }
  if(fits_keyword_get(result,"CRVAL1",value)){
    h->crval1 = atof(value);
  }
  else {
    printf("Can't read CRVAL1 in header\n");
    exit(-1);
  }
  if(fits_keyword_get(result,"CRVAL2",value)){
    h->crval2 = atof(value);
  }
  else {
    printf("Can't read CRVAL2 in header\n");
    exit(-1);
  }
  if(fits_keyword_get(result,"CD1_1",value)){
    h->cd1_1 = atof(value);
  }
  else {
    printf("Can't read CD1_1 in header\n");
    exit(-1);
  }
  if(fits_keyword_get(result,"CD2_2",value)){
    h->cd2_2 = atof(value);
  }
  else {
    printf("Can't read cd2_2 in header\n");
    exit(-1);
  }
  
  if(fits_keyword_get(result,"DATE-OBS",value)){
    int year,month,day;

    sscanf(value," '%d-%d-%d'",&year,&month,&day);
    
    h->dated = year+(month-1)/12.0+day/365.0;
  }
  else {
    printf("Can't read DATE-OBS in header\n");
    exit(-1);
  }

  fits_free(result);

}
/*****************************************************************/
void readSexfile(char* sexname,sexObj** mama,int* size,kw* h){
  FILE* sexfile = NULL;
  char line[1024];
  int nbElts = 0;
  sexObj* list = NULL;
  int size_list = 0;
 
  sexfile = fopen(sexname,"r");

  if(sexfile == NULL){
    printf("can't open sexfile %s\n",sexname);
    exit(-1);
  }

  /* find the size */
  while(fgets(line,sizeof(line),sexfile)!=NULL){
    size_list++;
  }

  /* allocate the memory */
  list = (sexObj*)(malloc(size_list*sizeof(sexObj)));

  rewind(sexfile);
  while(fgets(line,sizeof(line),sexfile)!=NULL){
    int status;
    /* read the line */
    status = sscanf(line,"%d %f %f %lf %lf %d  %lf  %lf  %lf  %lf",
      &list[nbElts].number,
      &list[nbElts].mag_auto,
      &list[nbElts].mag_err_auto,
      &list[nbElts].background,
      &list[nbElts].flux_max,
      &list[nbElts].isoarea_image,
      &list[nbElts].x_image,
      &list[nbElts].y_image,
      &list[nbElts].a_image,
      &list[nbElts].b_image);
    cd_xypix_to_radec(list[nbElts].x_image,list[nbElts].y_image,h->crval1,
      h->crval2,h->crpix1,h->crpix2,h->cd1_1,0.0,0.0,h->cd2_2,&list[nbElts].ra,
      &list[nbElts].dec);
    nbElts++;
  }
  *mama = list;
  *size = size_list;

  fclose(sexfile);

}
/*****************************************************************/


void readUcac2file(char* ucac2name,ucac2Obj** ucac2,int* size,kw* h){
  FILE* ucac2file = NULL;
  char line[1024];

  ucac2file = fopen(ucac2name,"r");
  int nbElts = 0;
  ucac2Obj* list = NULL;
  int i,size_list = 0;

  if(ucac2file == NULL){
    printf("can't open ucac2file %s\n",ucac2name);
    exit(-1);
  }

  /* find the size */
  while(fgets(line,sizeof(line),ucac2file)!=NULL){
    size_list++;
  }

  rewind(ucac2file);
  
  /* skip the four first lines */
  for(i=0;i<4;i++){
    fgets(line,sizeof(line),ucac2file);
  }
  size_list = size_list-6;
  /* allocate the memory */
  list = (ucac2Obj*)(malloc(size_list*sizeof(ucac2Obj)));

  while( (fgets(line,sizeof(line),ucac2file)!=NULL) && (nbElts<size_list)){
    /* read the line */
    sscanf(line," %s %lf%lf %*d %*d %*d %f %*d %*d %*d %lf %lf %lf %lf",
      list[nbElts].name, &list[nbElts].ra, &list[nbElts].dec,
      &list[nbElts].mag, &list[nbElts].epRa, &list[nbElts].epDec,
      &list[nbElts].pmRa, &list[nbElts].pmDec);
   /* compute ra and dec at epoch of plate */
   list[nbElts].decDate = list[nbElts].dec+(h->dated-2000.0)*
     list[nbElts].pmDec/1000.0/3600.0;

   list[nbElts].raDate = list[nbElts].ra+(h->dated-2000.0)*
     list[nbElts].pmRa/1000.0/3600.0/cos(0.5*(list[nbElts].decDate+list[nbElts].dec)*M_PI/180.0);

   nbElts++;
  }
  *ucac2 = list;
  *size = size_list;

  fclose(ucac2file);

}
/*****************************************************************/

#define        CROSS_SYNTAX \
"mcross      cross identification between sextractor file and ucac2 file\n\
               -image <fits image>\n\
	       -sex   <sextractor file>\n\
	       -ucac2 <ucac2 file>\n\
	       -output <output file>\n"

int main(int argc,char* argv[]){
  char* imagename = NULL;
  char* sexname = NULL;
  char* ucac2name = NULL;
  char* outputname = NULL;
  sexObj* mama = NULL;
  ucac2Obj* ucac2 = NULL;
  int mamaSize,ucac2Size;
  int i,j;
  kw* header;

  if(argc < 9){
    printf("%s",CROSS_SYNTAX);
    exit(-1);
  }
  for( i=1;i<argc;i++ ){
    if( strcmp("-image",argv[i])==0 ){
      imagename = strdup(argv[++i]);  
    }
    else if(strcmp("-sex",argv[i])==0 ){
      sexname = strdup(argv[++i]);
    }
    else if( strcmp("-ucac2",argv[i])==0 ){
      ucac2name = strdup(argv[++i]);
    }
    else if( strcmp("-output",argv[i])==0 ){
          outputname = strdup(argv[++i]);
    }
    else{
      printf("%s",CROSS_SYNTAX);
      exit(-1);
    }
  }
  /* check parameters */
  if (imagename == NULL){
    printf("ERROR invalid -image parameter\n");
    exit(-1);
  }
  
  if (sexname == NULL){
    printf("ERROR invalid -sex parameter\n");
    exit(-1);
  }

  if (ucac2name == NULL){
    printf("ERROR invalid -ucac2 parameter\n");
    exit(-1);
  }

  if (outputname == NULL){
    printf("ERROR invalid -output parameter\n");
    exit(-1);
  }

  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("MCROSS cross identification\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("Image file = %s\n", imagename);
  printf("Sextractor file  = %s\n", sexname);
  printf("Ucac2 file  = %s\n", ucac2name);
  printf("Output file  = %s\n", outputname);

  /* init */
  header = (kw*)(malloc(sizeof(kw)));
  mama = (sexObj*)(malloc(sizeof(sexObj*)));
  mamaSize = 0;

  ucac2 = (ucac2Obj*)(malloc(sizeof(ucac2Obj*)));
  ucac2Size = 0;
 
  /* get data */
  readKW(imagename, header);
  readSexfile(sexname, &mama, &mamaSize, header);
  readUcac2file(ucac2name,&ucac2,&ucac2Size,header);
  printf("UCAC2 : %d  | MAMA %d\n",ucac2Size,mamaSize);

  /* prepare cross id */
  double ramean,decmean,rasig,decsig,dmin,d,dmean,dsig;
  int nbfind = 0;

  ramean = decmean = rasig = decsig = dmean = dsig = 0.0;

  /* do the cross id */
  for(i=0;i<ucac2Size;i++){
    sexObj* nearest = NULL;
    double dif_ra,dif_dec;
    dmin = 9999;
    
    for(j=0;j<mamaSize;j++){
      d = distance(ucac2[i].raDate,ucac2[i].decDate,mama[j].ra,mama[j].dec);
      if (d < dmin){
        dmin = d;
	nearest = &mama[j];
      }
    }
    /* log the object if near than DMIN */
    dmin = dmin *3600;
    if (dmin <= DMIN){
      double decrad;
      nbfind++;
      dif_dec = (ucac2[i].decDate-nearest->dec)*3600.0;
      //printf("%f %f %f %f %f\n",ucac2[i].raDate,ucac2[i].decDate,nearest->ra,nearest->dec,dmin); 
      decrad = ucac2[i].decDate*DEG_TO_RAD;

      dif_ra = sin(decrad)*sin(decrad);
      dif_ra += cos(decrad)*cos(decrad)*cos((ucac2[i].raDate-nearest->ra)*DEG_TO_RAD);
      if(dif_ra > 1){
        dif_ra = 1.0;
      }
      dif_ra = acos(dif_ra)*RAD_TO_DEG*3600.0;
     if(ucac2[i].raDate < nearest->ra){
       dif_ra = -dif_ra;
     }
     dmean +=dmin;
     ramean += dif_ra;
     decmean += dif_dec;
     dsig +=dmin*dmin;
     rasig += dif_ra*dif_ra;
     decsig += dif_dec*dif_dec;
    }
  }
   if (nbfind > 0){
     dmean = dmean / nbfind;
     ramean = ramean / nbfind;
     decmean = decmean / nbfind;
   }
   if(nbfind > 1){
     dsig = dsig/(nbfind-1);
     rasig = rasig/(nbfind-1);
     rasig = sqrt(rasig);
     decsig = decsig/(nbfind-1);
     decsig = sqrt(decsig);
     dsig = sqrt(dsig);
  }
  
  printf("ucac2 find ramean decmean rasig decsig dmean dsig\n%d %d %.15f %.15f %.15f %.15f %.15f %.15f\n",ucac2Size,nbfind,ramean,decmean,rasig,decsig,dmean,dsig);
  /* free data */
  free(header);
  free(mama);
  free(ucac2);  
  /* free parameters */
  free(imagename);
  free(sexname);
  free(ucac2name);
  free(outputname);

  return 0;
}



