#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <getopt.h>
#include <fits.h>
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* SYNTAX
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

#define         MEXPLODE5_SYNTAX   \
"mexplode5      <explode a mosaic fits file in five file\n\
                -file \n\
		-size\n\
		-dest\n\
Example :\n\
  mexplode5 -file mymosaique.fits -size 1024 -dest ~/temp"


/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* sub_image
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int sub_image(fits* ref, double crpix1, double crpix2,int x, int y, int width, int height, char* name){
  char value[22];
  FILE* file = NULL;
  int nb = 0,i=0;
  int size_pixel = 0;
  int naxis1,naxis2;
  double ncrpix1,ncrpix2;
  PIXTYPE* line = NULL;
  int sum = 0;

  /* write naxis of the sub image */
  sprintf(value,"%20d\0",width);
  fits_keyword_set(ref,"NAXIS1",value);
  sprintf(value,"%20d\0",height);
  fits_keyword_set(ref,"NAXIS2",value);
  ncrpix1 = crpix1-x;
  ncrpix2 = crpix2-y;
  /* write th new position */
  sprintf(value,"%20E\0",ncrpix1);
  fits_keyword_set(ref,"CRPIX1",value);
  sprintf(value,"%20E\0",ncrpix2);
  fits_keyword_set(ref,"CRPIX2",value);  

  
  /* open the sub image file */
  file = fopen(name,"w");
  if( file == NULL ){ 
    printf("Can't write file %s\n",name);
    exit(-1);
  }
  
  /* write in the sub image file */ 
  nb = fwrite(ref->header, 2880 * ref->nb_header_block,1,file);

  if (nb!= 1) {
    printf("Can't write the header in the sub image file %s\n",name);
    exit(-1);
  }

  line = (PIXTYPE*)(malloc(sizeof(PIXTYPE)*width));
  size_pixel = sizeof(PIXTYPE);
  /* doing the sub image */
 
  /* go to the first line */
  fseek(ref->file,ref->nb_header_block*2880+x*size_pixel+y*size_pixel*ref->naxis1,SEEK_SET);

  nb = fread(line,width,size_pixel,ref->file);

  printf("name %s line %d\n",name , line[0]);
  if ( (line[0] == 128) || (line[width-1] == 128) ){
    free(line);
    fclose(file);
    return 1;
  }
  else{
    nb = fwrite(line,width,size_pixel,file);
    fseek(ref->file,(ref->naxis1-width)*size_pixel,SEEK_CUR);
    /* read the image */
    for(i=1; i < height-1;i++){
      nb = fread(line,width,size_pixel,ref->file);
      nb = fwrite(line,width,size_pixel,file);
      fseek(ref->file,(ref->naxis1-width)*size_pixel,SEEK_CUR);
    } 

    nb = fread(line,width,size_pixel,ref->file);
    if ( (line[0] == 128) || (line[width-1] == 128) ){
      free(line);
      fclose(file);
      return 1;
    }

    nb = fwrite(line,width,size_pixel,file);
    fseek(ref->file,(ref->naxis1-width)*size_pixel,SEEK_CUR);
    free(line);
    naxis1 = ref->naxis1;
    naxis2 = ref->naxis2;
    ref->naxis1 = width;
    ref->naxis2 = height;
    fits_complete_with_zero(ref,file);
    fclose(file);
    ref->naxis1 = naxis1;
    ref->naxis2 = naxis2;
    return 0;
  }
}

/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* mexplode5
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int main(int argc,char* argv[]){

  int i;
  fits* result = NULL;
  char* filename = NULL;
  char* dest = NULL;
  char* rootname = NULL;
  char imagename[128];
  double crpix1,crpix2;
  char value[22];

  int size = 1024;
  int x = 2048, y =2048;

   if(argc < 2){
      printf("%s",MEXPLODE5_SYNTAX);
     exit(-1);
   }
  for( i=1;i<argc;i++ ){
    if( strcmp("-file",argv[i])==0 ){
      filename = strdup(argv[++i]);  
    }
   else if( strcmp("-size",argv[i])==0 ){
     size = atoi(argv[++i]);
   }
   else if( strcmp("-dest",argv[i])==0 ){
     dest = strdup(argv[++i]);
   }
   else {
     printf("%s",MEXPLODE5_SYNTAX);
     exit(-1);
   }
 }

  if (filename == NULL){
    printf("You must provide a non null filename\n");
    exit(-1);
  }
  rootname = strtok(strdup(strrchr(filename,'/')),".");

  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("EXPLODE IN FIVE\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("File to explode = %s\n", filename);
  printf("dest = %s\n",dest);
  printf("Size = %d\n",size);

  result = fits_new(filename);
 /* read the reference system */
  if(fits_keyword_get(result,"CRPIX1",value)){
    crpix1 = atof(value);
  }
  else {
    printf("Can't read CRPIX1 in header\n");
    exit(-1);
  }
  if(fits_keyword_get(result,"CRPIX2",value)){
    crpix2 = atof(value);
  }
  else {
    printf("Can't read CRPIX2 in header\n");
    exit(-1);
  }

  /* verify the parameters */
  if(result->bitpix != 16){
    printf("Can't explode this file bitpix != 16\n");
    exit(-1);
  }
  /* go to the pixel */
  fits_seek_pixels(result);

  /* cut five sub images */
  x = 2048;
  sprintf(imagename,"%s/%s_explode_1.fits",dest,rootname);
  while ( sub_image(result,crpix1,crpix2,x,x,size,size,imagename) ){
    x += 2048;
  }

  x = 2048;
  sprintf(imagename,"%s/%s_explode_2.fits",dest,rootname);
  while (sub_image(result,crpix1,crpix2,x,result->naxis2-x-size,size,size,imagename)){
    x+=2048;
  }

  x = 2048;
  sprintf(imagename,"%s/%s_explode_3.fits",dest,rootname);
  while(sub_image(result,crpix1,crpix2,result->naxis1-x-size,result->naxis2-x-size,size,size,imagename)){
    x += 2048;
  }

  x = 2048;
  sprintf(imagename,"%s/%s_explode_4.fits",dest,rootname);
  while(sub_image(result,crpix1,crpix2,result->naxis1-x-size,x,size,size,imagename)){
    x+=2048;
  } 

  x = 2048;
  sprintf(imagename,"%s/%s_explode_5.fits",dest,rootname); 
  sub_image(result,crpix1,crpix2,rint(result->naxis1/2)-size/2,rint(result->naxis2/2)-size/2,size,size,imagename);
  
  fits_free(result);
  free(rootname);
  free(filename);
  return 0;
}
