#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <mosreg.h>
#include <math.h>
#include <getopt.h>

/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* crop_mode
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

#define         CROP_SYNTAX \
"mosreg -crop                         \n\
                -file   <file to crop>\n\
		-output <result file>\n\
Example :\n\
  mosreg -crop -file mymosaique.fits -ouput crop.fits"


/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* crop_mode
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int crop_mode(int argc,char* argv[]){

  short* buff16 = NULL;
  long nb_zero_pixels = 0;
  long top_line = 0;
  long bottom_line = 0;
  long left_col = 0;
  long right_col = 0;
  long nbzero_pixels,oldnaxis1;
  long i,j,k;
  char* filename = NULL,* resultname  = NULL;
  char value[22];
  fits* result = NULL;
  FILE* file = NULL;
  int nbread;
  double crpix1,crpix2;
 
   if(argc < 6){
      error(-1,CROP_SYNTAX);
  }
  for( i=2;i<argc;i++ ){
    if( strcmp("-file",argv[i])==0 ){
      filename = strdup(argv[++i]);  
    }    
    else if(strcmp("-output",argv[i])==0 ){
      resultname = strdup(argv[++i]);       
    }
    else{
      error(-1,CROP_SYNTAX);
    }
  }  
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("CROP MODE\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("File to crop = %s\n", filename);
  printf("Result       = %s\n", resultname);
  
  result = fits_new(filename);
 /* verify the parameters */
  if(result->bitpix != 16){
    error(-1,"Can't crop this file bitpix != 16");
  }
  /* go to the pixel */
  fits_seek_pixels(result);  

  /* search the line with zero pixel*/
  buff16 = (short*)(malloc(result->naxis1*sizeof(short)));
 

  /* find top line */
  for (i=0;i<result->naxis2;i++){
    nbread = fread(buff16,result->naxis1,sizeof(short),result->file);
    nbzero_pixels = 0;
    swap16((PIXTYPE*)buff16,result->naxis1);

    for(j=0;j<result->naxis1;j++){
      if( buff16[j]== -32768 ){
        nbzero_pixels++;
      }
      else break;
    }
    if(nbzero_pixels != result->naxis1){
      top_line = i;
      break;
    }
  }

  /* find left_col and right col */
  left_col = result->naxis1;
  for(i=top_line;i<result->naxis2;i++){
    k =0;
    for(j=0;j<result->naxis1;j++){
      if( buff16[j]== -32768 ){
          k ++;
      }
      else break;
    }
    //printf("i %d k %d naxis1 %d\n",i,k,result->naxis1);
    if(k == result->naxis1){
      bottom_line = i;
      break;
    }
    if(left_col>k){
      //      printf("i %d lef_col %d k %d\n",left_col,i,k);
      left_col = k;
    }
    k = result->naxis1-1;
    for(j=result->naxis1-1;j>=0;j--){
      if( buff16[j]== -32768 ){
        k--;
      }
      else break;
    }
    if(right_col<k){
      right_col = k;
    }
    nbread = fread(buff16,result->naxis1,sizeof(short),result->file);
    swap16((PIXTYPE*)buff16,result->naxis1);
  }
  printf("top %d bottom %d left %d right %d\n",top_line,bottom_line,left_col,right_col);
  /* write new naxis1 and naxis2 */
  oldnaxis1 = result->naxis1;
  result->naxis1 = right_col-left_col;
  result->naxis2 = bottom_line-top_line;
  sprintf(value,"%20d\0",result->naxis1);
  fits_keyword_set(result,"NAXIS1",value);
  sprintf(value,"%20d\0",result->naxis2);
  fits_keyword_set(result,"NAXIS2",value);

  /* read the reference system */
  if(fits_keyword_get(result,"CRPIX1",value)){
    crpix1 = atof(value);
  }
  else {
    error(-1,"Can't read CRPIX1 in header");
  }
  if(fits_keyword_get(result,"CRPIX2",value)){
    crpix2 = atof(value);
  }
  else {
    error(-1,"Can't read CRPIX2 in header");
  }

  /* compute the new position of reference pixel */
  crpix1 = crpix1-left_col;
  crpix2 = crpix2-top_line+1;

  /* write this new position */
  sprintf(value,"%20E\0",crpix1);
  fits_keyword_set(result,"CRPIX1",value);
  sprintf(value,"%20E\0",crpix2);
  fits_keyword_set(result,"CRPIX2",value);  

  
  /* write the result file */
  file = fopen(resultname,"w");
  if( file == NULL ){ 
    error(-1,"Can't write in result file");
  } 
  /* write the header */
  nbread = fwrite(result->header, 2880 * result->nb_header_block,1,file);
  if (nbread != 1) {
    error(-1,"Can't write the header in the convert file");
  }

  /* go to the pixel */
  fits_seek_pixels(result);
  
  /* go to the first line */
  fseek(result->file,sizeof(short)*(top_line-1)*oldnaxis1,SEEK_CUR);
  
  /* write the pixel */
  for (i=0;i<result->naxis2;i++){
    nbread = fread(buff16,oldnaxis1,sizeof(short),result->file);
    nbread = fwrite(&buff16[left_col],result->naxis1,sizeof(short),file);
  }
  
  /* complete with zero */
  fits_complete_with_zero(result,file);
  free(filename);
  free(resultname);
  free(buff16);
  fclose(file);
  fits_free(result);
}
