#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <time.h>
#include "mosreg.h"


/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* parse_value
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
static void parse_value(FILE* file,char* value){
  char c;
  int i = 0;
  
  c = fgetc(file);
  
  while(c != '['){
    c = fgetc(file);
  }
  c = fgetc(file);
  while( (c != ']') && i<71){
    value[i] = c;
    i++;
    c = fgetc(file);
  }
  value[i]='\0';
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* parse_comment
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
static void parse_comment(FILE* file,char* comment){
  char c;
  int i = 0;
  
  c = fgetc(file);
  
  while(c != '/'){
    c = fgetc(file);
  }
  c = fgetc(file);
  while( (c != '\n') && i<71){
    comment[i] = c;
    i++;
    c = fgetc(file);
  }
  comment[i]='\0';
}
/*
----------------------------------------------------------------------------------------
*/
/*
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* header_new
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
header* header_new(char* file,pave* pav1, int nbX,int nbY,int compX, int compY,
  char* filename, int numFirst, int numLast,atlas* atlas1,wcs* wcs1){
  
  header* result = NULL;
  char record[80];
  char * line = NULL;
  int i,j,len,lc;
  
  result = (header*)(malloc(sizeof(header)));
  
  result->nbKeys = 0;
  result->nbX = nbX;
  result->nbY = nbY;
  result->pavRef = pav1;
  result->wcsRef = wcs1;
  result->compx = compX;
  result->compy = compY;
  result->sizeX = result->pavRef->sizeX / result->compx * nbX;
  result->sizeY = result->pavRef->sizeY / result->compy * nbY;
  
  result->filename = (char*)(malloc(sizeof(char)*256));
  /* compute mosaique filename */
  if(filename == NULL){
    char* name = strrchr(pav1->path,'/');
    if (name == NULL){
      name=pav1->path;
    }
    else{
      name = name+1;
    }
    sprintf(result->filename,"%s_%i_%i_%i_mosreg.fits",name,numFirst,numLast,compX * compY);
  }
  else{
    sprintf(result->filename,"%s",filename);
  }
  
  if(file != NULL){
    result->file = fopen(file,"r");
    if (result->file == NULL) {
      error(-1,"Impossible d'ouvrir le fichier header");
    }
    while (fgets(record,1024,result->file)){
      result->nbKeys++;
    }
    /* allocate the memory */
    result->nb_header_blocks = (result->nbKeys)/36;
    if( result->nbKeys%36 ) 
      result->nb_header_blocks++;
    result->header = (char*)calloc(result->nb_header_blocks*2880,sizeof(char));
    if( result->header == NULL ){
      error(-1,"Pas assez de mmoire pour crer le header de la mosaique");
    }
    /* read the file */
    rewind(result->file);
    
    for(i=0;i<result->nbKeys-1;i++){
      char keyword[9];
      char command[72];
      char value[72];
      char comment[72];
      char c;

      for(j=0;j<8;j++){
        keyword[j]=fgetc(result->file);
      }
      keyword[8] = '\0';
      parse_value(result->file,command);
      /* find the command */
      if ( strncmp(command,"compute",7)==0 ){
        parse_comment(result->file,comment);
	/* analyse the keyword */
	if( strncmp(keyword,"NAXIS1",6)==0 ){
	  result->naxis1 = pav1->sizeY*nbY/result->compy;
	  sprintf(value,"%20d\0",result->naxis1);
	}
	else if( strncmp(keyword,"NAXIS2",6)==0 ){
	  result->naxis2 = pav1->sizeX*nbX/result->compx;
	  sprintf(value,"%20d\0",result->naxis2);
	}
	else if( strncmp(keyword,"INCR1",5)==0 ){
	  result->incr1 = 10*compY;
	  sprintf(value,"%20d\0",result->incr1);
	}
	else if( strncmp(keyword,"INCR2",5)==0 ){
	  result->incr2 = -10*compX;	  
	  sprintf(value,"%20d\0",result->incr2);
	}
	else if( strncmp(keyword,"FSTPAV",6)==0 ){
	  sprintf(value,"'%s'\0",pav1->fstpav);
	}
	else if( strncmp(keyword,"FILENAME",8)==0 ){
	  sprintf(value,"'%s'\0",result->filename);
	}
	else if( strncmp(keyword,"FLD",3)==0 ){
	  sprintf(value,"'%s'\0",atlas1->fld);
	}
	else if( strncmp(keyword,"CNPIX1",6)==0 ){
	  result->cnpix1 = (pav1->num/pav1->nxpavmax)*result->sizeY/nbY;
          sprintf(value,"%20i\0",result->cnpix1);
	}
	else if( strncmp(keyword,"CNPIX2",6)==0 ){
	  result->cnpix2 = (pav1->nxpavmax+1-pav1->num-nbX)*result->sizeX/nbX;
	  sprintf(value,"%20i\0",result->cnpix2);  
	}
	else if( strncmp(keyword,"XPIXELSZ",8)==0 ){
	  sprintf(value,"%20d\0",10*compX);
	}
	else if( strncmp(keyword,"YPIXELSZ",8)==0 ){
	  sprintf(value,"%20d\0",10*compY);
	}
	else if( strncmp(keyword,"DATE",4)==0 ){
	  time_t t;
          struct tm *tb;
	  
          t = time(NULL);
          tb = localtime(&t);
	  sprintf(value,"'%04d-%02d-%02dT%02d:%02d:%02d'",1900+tb->tm_year,tb->tm_mon+1,
	    tb->tm_mday,tb->tm_hour,tb->tm_min,tb->tm_sec);
	}
	else if( strncmp(keyword,"PPO3",4)==0 ){
	  result->ppo3 = wcs1->ycz *1000.0 - pav1->yoffset;
	  sprintf(value,"%20E\0",result->ppo3);
	}
	else if( strncmp(keyword,"PPO6",4)==0 ){
	  result->ppo6 = pav1->xoffset - wcs1->xcz *1000.0 + pav1->nxpavmax*10240;
	  sprintf(value,"%20E\0",result->ppo6);
	}
	else {
	  printf("can't compute the value of the keyword %s\n",keyword);
	  exit(-1);
	}
      }
      else if( strncmp(command,"value",5)==0 ){
	 /* read value */
	 parse_value(result->file,value);
	 /* read comment */
	 parse_comment(result->file,comment);
      }
      else if( strncmp(command,"pave",4)==0 ){
	 int cut;
	 /* read value */
	 parse_value(result->file,value);
	 /* read comment */
	 parse_comment(result->file,comment);
	 /* analyse the value */
	 pave_keyword_get(pav1,value,value);
	 cut = strlen(value)-1;
	  if (value[cut] == ' '){
	    value[cut] = 0;
	  }
      }
      else if( strncmp(command,"plates",6)==0 ){
      	 /* read value */
	 parse_value(result->file,value);
	 /* read comment */
	 parse_comment(result->file,comment);
	 /* analyse the keyword */
	if( strncmp(value,"PLTLABEL",8)==0 ){
	  sprintf(value,"%s\0",atlas1->pltlabel);
	}
	else if( strncmp(value,"EMULSION",8)==0 ){
	  sprintf(value,"%s\0",atlas1->emulsion);
	}
	else if( strncmp(value,"FILTER",6)==0 ){
	  sprintf(value,"%s\0",atlas1->filter);
	}
	else if( strncmp(value,"ALPHA_50",8)==0 ){
	  sprintf(value,"%s\0",atlas1->alpha_50);
	}
	else if( strncmp(value,"DELTA_50",8)==0 ){
	  sprintf(value,"%s\0",atlas1->delta_50);
	}
	else if( strncmp(value,"DATE-OBS",8)==0 ){
	  sprintf(value,"%s\0",atlas1->dateobs);
	}
	else if( strncmp(value,"JDAY",4)==0 ){
	  sprintf(value,"%s\0",atlas1->jday);
	}
	else if( strncmp(value,"MJDAY_S",7)==0 ){
	  sprintf(value,"%s\0",atlas1->mjday_s);
	}
	else if( strncmp(value,"MJDAY_M",7)==0 ){
	  sprintf(value,"%s\0",atlas1->mjday_m);
	}
	else if( strncmp(value,"LST_SOE",7)==0 ){
	  sprintf(value,"%s\0",atlas1->lst_soe);
	}
	else if( strncmp(value,"PLTDATE",7)==0 ){
	  sprintf(value,"%s\0",atlas1->pltdate);
	}
	else if( strncmp(value,"EXPOSURE",8)==0 ){
	  sprintf(value,"%s\0",atlas1->exposure);
	}
	else if( strncmp(value,"PLTGRADE",8)==0 ){
	  sprintf(value,"%s\0",atlas1->pltgrade);
	}
	else {
	  printf("can't compute the value of the keyword %s\n",keyword);
	  exit(-1);
	}
      }
      else if( strncmp(command,"wcs",3)==0 ){
	int end;
        /* read value */
	parse_value(result->file,value);
	/* read comment */
	parse_comment(result->file,comment);
	/* analyse value */
	wcs_keyword_get(wcs1,value,value);
	end = strlen(value);
	if (value[end-1] ==' '){
	  value[end-1] = 0;
	}
      }
      else {
        error(-1," header file is not valid");
      }
      line = &result->header[i*80];
      
      /* write the header */
      for(j=0;j<8;j++){
        line[j]=keyword[j];
      }
      line[8] = '=';
      line[9] = ' ';
      len = strlen(value);
      
      for(j=0;j<len;j++){
        line[j+10]=value[j];
      }

      while(len < 20){
        line[len+10] = ' ';
	len++;
      }
      line[len+10] = ' ';
      line[len+11] = '/';
      lc = strlen(comment);
      for(j=0;j<lc;j++){
        line[j+12+len]=comment[j];
      }
      for(j=len+12+lc;j<80;j++){
        line[j]=' ';
      }
    }
    line = &result->header[(result->nbKeys-1)*80];
    line[0] = 'E';
    line[1] = 'N';
    line[2] = 'D';    
    for(j=3;j<80;j++){
      line[j] =' ';
    }
    for(j=result->nbKeys*80;j<result->nb_header_blocks*2880;j++){
      result->header[j] =' ';
    }
    fclose(result->file);
  }
  return result;
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* header_free
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void header_free(header* headerfile){
  if(headerfile){
    free(headerfile->header);
    free(headerfile);
  }
}
/*
----------------------------------------------------------------------------------------
*/
