#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>

#include "mosreg.h"

/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* SYNTAX definitions
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
#define		MOSAIC_SYNTAX \
"mosreg                                          \n\
		-i < radical du nom des paves [REQUIRED]>\n\
                -d < indice de la premiere image [defaut=1]>\n\
		-m < nb. de chiffres pour le no. du pave [defaut=3] >\n\
		-n < taux de compression  [defaut=1] >\n\
		-x < nb. de paves en Xmama [defaut=28]>\n\
		-y < nb. de paves en Ymama [defaut=28]>\n\
		-c < cree une mosaique centre sur un pave [<pave> <radius>] >\n\
		-h < header file name>\n \
		-w < wcs file name > \n\
                -a < atlas file name > \n\
		-o < mosaic output file name >\n"

/*
void center(mamastruct * mama, mosaiquefilestruct * path, int radius)
{
        int modulo, div;
	int reduction;

	modulo = (path->numpaves -1) % mama->taille_totale_X;
	div = (path->numpaves - 1) / mama->taille_totale_X;
	printf("x %i \n",mama->nb_paves_X);
	printf("y %i \n",mama->nb_paves_Y);
	printf("modulo %i \n",modulo);
	printf("div %i \n",div);
	
	printf("modulo + radius %i \n",modulo + radius);
	if ((modulo + radius) >= mama->taille_totale_X)
	{
	    reduction = (modulo + radius) -  mama->taille_totale_X + 1;
	    mama->nb_paves_X = mama->nb_paves_X - reduction;
	}
	printf("x %i \n", mama->nb_paves_X);
	printf("modulo - radius %i \n",modulo - radius);
	if ((modulo - radius) < 0)
	{
	    reduction = -(modulo - radius);
	    mama->nb_paves_X = mama->nb_paves_X - reduction;
	    modulo = 0;
	}
	else modulo = modulo - radius;
	printf("x %i \n", mama->nb_paves_X);
	printf("div + radius %i \n",div + radius);
	if ((div + radius) >= mama->taille_totale_Y)
	{
	    reduction = (div + radius) - mama->taille_totale_Y + 1;
	    mama->nb_paves_Y = mama->nb_paves_Y - reduction;
	}
	printf("y %i \n", mama->nb_paves_Y);
	printf("div - radius %i \n",div - radius);
	if ((div - radius) < 0)
	{
	    reduction = -(div - radius);
	    mama->nb_paves_Y = mama->nb_paves_Y - reduction;
	    div = 0;
	}
	else div = div - radius;
	printf("y %i \n", mama->nb_paves_Y);
	path->indice_depart = div * mama->taille_totale_X + modulo + 1;
	printf("idep %i \n",path->indice_depart );
}
*/

int main(int argc,char* argv[]){
  mosaique* result = NULL;
  pave* first = NULL;
  wcs* wcsname = NULL;
  atlas* atlasname = NULL;
  header* headername = NULL;
  char* pavePath = NULL;
  int sizeNum,numFirst,numLast;
  int compx,compy,nbX,nbY;
  int i;
  int radius;
  char* filename = NULL;
  char* wcsfile = NULL;
  char* headerfile = NULL;
  char* atlasfile = NULL;

  /* init */
  printf("==========================================================\n");
  printf("                    MOSREG\n");
  printf("==========================================================\n");
  //path.indice_depart = 1;
  /* number of digits in pave name */
  sizeNum = 3;
  /* number of the first pave */
  numFirst = 1;
  nbX = 28;
  nbY = 28;
  compx = 1;
  compy = 1;
  if(argc < 2){
      error(-1,MOSAIC_SYNTAX);
  }
  for( i=1; i<argc; i++){
    switch( tolower(argv[i][1]) ){
    
      /* REQUIRED argument */
    
      /* path where to find pave */
      case 'i':
        pavePath = (char*)(malloc(sizeof(char)*256)); 
        sprintf(pavePath, "%s", argv[++i]);
        break;
	
      /* OPTIONAL argument */
    
      /* First pave number */
      case 'd':
        numFirst = atoi(argv[++i]);
      break;
    
      /* number of digits in the name of pave */  
      case 'm': 
        sizeNum = atoi(argv[++i]);
      break;
     
      /* compress rate */
      case 'n': 
        compx = atoi(argv[++i]);
        compy = compx;
      break;
     
      /* nb paves in xmama */
      case 'x': 
        nbX = atoi(argv[++i]);
      break;
     
      /* nb paves in ymama */
      case 'y': 
        nbY = atoi(argv[++i]);
      break;
     
      /* name of the result file */
      case 'o':
        filename = strdup(argv[++i]);
      break;

      /* name of the wcs file */
      case 'w':
        wcsfile = strdup(argv[++i]);
      break;

      /* name of the header file */
      case 'h':
        headerfile = strdup(argv[++i]);
      break;

      /* name of the atlas file */
      case 'a':
        atlasfile = strdup(argv[++i]);
      break;

      default : error(1,MOSAIC_SYNTAX);
    }
  }
  if (pavePath == NULL){
    error(-1,"Vous devez spcifier un chemin pour les pavs avec l'option -i");
  }
  if (headerfile == NULL){
    error(-1,"Vous devez spcifier un fichier header avec l'option -h");
  }
  if (atlasfile == NULL){
    error(-1,"Vous devez spcifier un fichier atlas avec l'option -a");
  }
  /* create the first pave */
  first = (pave*)pave_new(pavePath,sizeNum,numFirst);
  free(pavePath);
  /* verify nbX and nbY */
  if( (nbX<=0) || (nbX>first->nxpavmax)){
    error(-1,"Nombre de pave en X mama incorrect");
  }
  if( (nbY<=0) || (nbY>first->nypavmax)){
    error(-1,"Nombre de pave en Y mama incorrect");
  }
  
  /* find the number of last pave */     
  numLast = numFirst+nbX-1+(nbY-1)*first->nxpavmax; 
/*
	
	if (path.indice_depart != path.numpaves)
	{	
	  center(&mama, &path, radius);
	}
	// indice de dpart
	if ((path.indice_depart < 1) || 
            (path.indice_depart > mama.taille_totale_Y * mama.taille_totale_X))
	{
	       printf("indice de dpart invalide : default => 1 \n");
	       path.indice_depart = 1;
        }
        // nombre de pavs en X
	if ((mama.nb_paves_X  < 1) ||
            (mama.nb_paves_X > mama.taille_totale_X))
	{
               printf("nombre de paves Xmama invalide : default => 1 \n");
	       mama.nb_paves_X = 1;
        }
        // nombre de pavs en Y
	if ((mama.nb_paves_Y  < 1) ||
            (mama.nb_paves_Y > mama.taille_totale_Y))
	{
               printf("nombre de paves Ymama invalide : default => 1 \n");
	       mama.nb_paves_Y = 1;
        }

	// indice de fin
	indice_fin = path.indice_depart + (mama.nb_paves_Y - 1) *  mama.taille_totale_X + mama.nb_paves_X - 1;
	if ((indice_fin < path.indice_depart) || 
            (indice_fin > mama.taille_totale_Y * mama.taille_totale_X))
	{
	       printf("indice de dpart invalide : default => max \n");
	       indice_fin = mama.taille_totale_Y * mama.taille_totale_X;
        }
*/
   
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("PAVES\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("Racine = %s\n", first->path);
  printf("Premier --> dernier = %d --> %d\n",numFirst,numLast);
  printf("Nb en (X,Y) mama = (%d,%d)\n",nbX,nbY);
  printf("Taille (NAXIS1, NAXIS2)= (%d,%d)\n", first->sizeX,first->sizeY);
  
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("COMPRESSION\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf ("Xmama = 1 / %d\n",compx);
  printf ("Ymama = 1 / %d\n",compy);

  wcsname = wcs_new(wcsfile);
 
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("WCS\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf ("Nombre de KW = %d\n",wcsname->nbKeys);

  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("ATLAS\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n"); 
  atlasname = atlas_new(first->path,atlasfile);

  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("HEADER\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");

  headername = header_new(headerfile,first,nbX,nbY,compx,compy,filename,
    numFirst,numLast,atlasname,wcsname);

  printf ("Nombre de KW = %d\n",headername->nbKeys);

  result = mosaique_new(first,headername, mirror_mode);

  header_free(headername);
  atlas_free(atlasname); 
  
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("RESULTAT\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");	 
  printf ("Nom du fichier = %s\n", result->filename);
  
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("PROCESSING\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("Nombre total d'images = %d\n", nbX*nbY);

  mosaique_image(result);
  
  mosaique_free(result);
  wcs_free(wcsname);
  pave_free(first);
  printf("==========================================================\n");
  printf("                    MOSREG ending successfully\n");
  printf("==========================================================\n");  
  return 0;
}
