#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>

#include "mosreg.h"

/*
----------------------------------------------------------------------------------------
*/
/*
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* mosaique_new
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
mosaique* mosaique_new(pave* pav1,header* head, int mirror_mode){

  header* header = NULL;
  int nbread;
  mosaique* mos = NULL;
  
  mos = (mosaique*)(malloc(sizeof(mosaique)));
  if(mos == NULL){
    error(-1,"Pas assez de mmoire pour crer la mosaique");
  }
  mos->filename=head->filename;
  mos->nbX = head->nbX;
  mos->nbY = head->nbY;
  mos->pavRef = pav1;
  mos->compx = head->compx;
  mos->compy = head->compy;
  mos->sizeX = head->sizeX;
  mos->sizeY = head->sizeY;
  mos->wcsRef = head->wcsRef;
  mos->mirror = mirror_mode;
  mos->file = fopen(mos->filename,"w");
  if( mos->file == NULL ){ 
    error(-1,"Ouverture impossible en ecriture du fichier mosaique");
  }

  /* write in the mos file */ 
  nbread = fwrite(head->header, 2880 * head->nb_header_blocks,1,mos->file);
  if (nbread != 1) {
    error(-1,"L'ecriture du header de la mosaique a chou");
  }
  return mos;
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* mosaique_free
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void mosaique_free(mosaique* mos){
  if(mos){
    fclose(mos->file);
    free(mos->filename);
    free(mos);
  }
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* mosaique_line
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void mosaique_line( int x, mosaique* mos, int nbX, int nbY, PIXTYPE* line){
  int y;

  /* iterate on Ymama (E-W)*/
  for(y=0; y < nbY; y++){
    /* numero du pave */
    int numPave;
    int i, j, k, l;
    int subx, suby;
    int pos_dep,indice_dep;	
    int sizeX;
    int sizeY;

    int moyenne;
    int modulo;
    int nbmerge;
    
    /* load pave */
    numPave = y * mos->pavRef->nxpavmax + x + mos->pavRef->num;
    pave_image(mos->pavRef,numPave);
    
    /* swap pixels if need */
    if( mos->compx !=1 ){
      swap16(mos->pavRef->pixels,mos->pavRef->sizeX*mos->pavRef->sizeY);
    }
    /* write a line */
    pos_dep = y * mos->pavRef->sizeY / mos->compy;
   
    nbmerge = mos->compx * mos->compy;
    sizeY = mos->pavRef->sizeY /mos->compy;
    sizeX = mos->pavRef->sizeX /mos->compx;  

    /* iterate on  X (N-S)*/
    for( i=0;i< sizeX;i++ ){
      subx = i * mos->compx;
      
      /* iterate on Y(E-W) */
      for (j = 0; j < sizeY; j++){
        suby = j * mos->compy;
        moyenne = mos->pavRef->pixels[i*sizeX +j];
        if (mos->compx != 1){	
          moyenne = 0;
	  for (k = subx; k < subx + mos->compx; k++){
	    for(l = suby; l < suby + mos->compy; l++){
	      moyenne = moyenne +(int)mos->pavRef->pixels[k * mos->pavRef->sizeY + l];
	    }
	  }	
	  modulo = moyenne % nbmerge;
	  modulo = ((modulo * 2) > moyenne);
	  moyenne = moyenne / nbmerge + modulo;
        }
        if(mos->wcsRef->nbKeys > 0){
          line[pos_dep+ (sizeX-1-i) * mos->sizeY + j] =(PIXTYPE)moyenne;
        }
        else{
          line[pos_dep+i*mos->sizeY+j] =(PIXTYPE)moyenne;
        }
      }
    }
  }
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* mosaique_image
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void mosaique_image(mosaique *mos){
  PIXTYPE * line = NULL;
  char * zero = NULL;
  long size;
  int i, size_to_complete;
  int nbwrite;
  int nb_block;
  int nbX,nbY;
  
  /* size of mosaique */
  nbX = mos->nbX;
  nbY = mos->nbY;

  /* allocate one line of mosaique */
  size = mos->sizeY * mos->pavRef->sizeX / mos->compx * sizeof(PIXTYPE);
  
  nb_block = (size * mos->nbX) /2880;
  if( (size * mos->nbX) % 2880 ){
    nb_block++;
  }

  line = (PIXTYPE*)(malloc(size));
  if(line == NULL){
    error (-1,"*ERREUR*:  memoire insuffisante pour une ligne ");
  }
  
  /* parcours des paves en Xmama (N --> S) */

  if (mos->wcsRef->nbKeys > 0 || (mos->mirror_mode == 1)){
    for(i= nbX-1; i >= 0; i--){
      mosaique_line(i, mos, nbX,nbY, line);
      printf ("ECRITURE MIRROR: ligne no %d / %d\n",nbX-i,nbX);
      if( mos->compx!=1 ){
        swap16(line,size/sizeof(PIXTYPE));
      }
      nbwrite = fwrite(line,size, 1,mos->file);
      if (nbwrite != 1) {
        printf(" PB. ecriture de la ligne en XMAMA\n");
        free(line);
        mosaique_free(mos);
        exit(-1);
      }
    }
  }
  else{
    for(i= 0; i < nbX; i++){
      mosaique_line(i,mos,nbX,nbY,line);
      printf ("ECRITURE : ligne no %d / %d\n",i+1,nbX);
      if( mos->compx!=1 ){
        swap16(line,size/sizeof(PIXTYPE));
      }
      nbwrite = fwrite(line,size, 1,mos->file);
      if (nbwrite != 1) {
        printf(" PB. ecriture de la ligne en XMAMA %d\n");
        free(line);
        mosaique_free(mos);
        exit(-1);
      }
     }
  }
 
  size_to_complete = nb_block * 2880 - (size * mos->nbX);
  zero = (char*)(calloc(size_to_complete,sizeof(char)));
  nbwrite = fwrite(zero,size_to_complete, 1,mos->file);
  if (nbwrite != 1) {
    printf(" PB. ecriture des zeros de fin de fichier fits\n");
    free(zero);
    mosaique_free(mos);
    exit(-1);
  }
  free(zero);
}
/*
----------------------------------------------------------------------------------------
*/
