#ifndef MOSREG_H
#define MOSREG_H

#include "config.h"

#include <string.h>
#include <fits.h>
/*****************************************************************************/
/*	DEFINITION DES TYPES                                                 */
/*****************************************************************************/

/* pave */
typedef struct
{
	int nbKeys;
	int sizeX;
	int sizeY;
	int nxpavmax;
	int nypavmax;
	float cdelt1;
	float cdelt2;
	int xoffset;
	int yoffset;
	int num;
	FILE *file;
	char* fstpav;
	char* path;
	char* format;
	long pixels_size;
	PIXTYPE *pixels;
	char *header;
	int nb_header_block;
} pave;

/* wcs */
typedef struct
{
	FILE* file;
	float ycz;
	float xcz;
	char* header;
	int nbKeys;
} wcs;

/* atlas */
typedef struct
{
  char fld[6];
  char pltlabel[30];
  char emulsion[30];
  char filter[30];
  char alpha_50[30];
  char delta_50[30];
  char dateobs[30];
  char jday[30];
  char mjday_s[30];
  char mjday_m[30];
  char lst_soe[30];
  char pltdate[30];
  char exposure[30];
  char pltgrade[30];
} atlas;

/* header */
typedef struct
{
	FILE* file;
	int naxis1;
	int naxis2;
	int incr1;
	int incr2;
	char* filename;
	int cnpix1;
	int cnpix2;
	float ppo3;
	float ppo6;
	char* header;
	int nb_header_blocks;
	int nbX;
	int nbY;
	int compx;
	int compy;
	int sizeX;
	int sizeY;
	pave* pavRef;
	wcs* wcsRef;
	int nbKeys;
} header;

/* mosaique */
typedef struct
{
	FILE *file;
	char* filename;
	int cnpix1;
	int cnpix2;
	int sizeX;
	int sizeY;
	int nbX;
	int nbY;
	int nbKeys;
	int compx;
	int compy;
	pave* pavRef;
	wcs* wcsRef;
	int mirror;
} mosaique;


/******************************************************************************/
/*      PROTOTYPES                                                            */
/******************************************************************************/
/******************************************************************************/
/*      wcs                                                                   */
/******************************************************************************/
wcs* wcs_new(char* file);
void wcs_free(wcs* wcsfile);
int wcs_keyword_get(wcs* wcsfile,char* keyword,char* value);
/******************************************************************************/
/*      atlas                                                                   */
/******************************************************************************/
atlas* atlas_new(char* filename,char* atlasfile);
void atlas_free(atlas* atlasfile);
/******************************************************************************/
/*      header                                                                */
/******************************************************************************/
header* header_new(char* file,pave* pav1,int nbX,int nbY,int compx,int compy,
  char* filename, int num1, int last,atlas* atlas1,wcs* wcs1);
void header_free(header* headerfile);
/******************************************************************************/
/*      mosaique                                                              */
/******************************************************************************/
mosaique* mosaique_new(pave* pav1,header* head);
void mosaique_pave_write(mosaique* mos, PIXTYPE* line,int posY);

/******************************************************************************/
/*      compression                                                           */
/******************************************************************************/
extern void creer_colonne();                                                                                                                                                             
/******************************************************************************/
/*      erreurs                                                               */
/******************************************************************************/
extern void error(short num, char * err);

/******************************************************************************/
/*      paves                                                                 */
/******************************************************************************/
pave* pave_new(char* filepath,int sizeNum, int numPav);
void pave_free(pave* pav);
void pave_image(pave* pav,int numPav);
int pave_get_keyword(pave* pav,char* keyword,char* value);
int pave_keyword_set(pave* pav,char* keyword,char* value);

#endif
