#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>

#include "mosreg.h"

/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* pave_new
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
pave* pave_new(char* filepath,int sizeNum, int numPav){
  int end = 0;
  int i;
  int nread;
  char filename[256];
  char value[22];
  pave* new = NULL;
  char* ptr = NULL; 
   
  new = (pave*)(malloc(sizeof(pave)));
  if( new == NULL ){
    error(-1,"Pas assez de mmoire pour crer un pav");
  }
  
  /* construct the format */
  new->path = strdup(filepath);
  ptr = strrchr(new->path,'/');
  new->fstpav = strdup(ptr+1);
  printf("fstpav %s\n",new->fstpav);
  new->num = numPav;
  new->format = (char*)(malloc(20*sizeof(char)));
  sprintf(new->format,"%%s%%.%id.fits",sizeNum);
  /* construct the filename */
  sprintf(filename,new->format,new->path,numPav);;  
  /* open the file in read */
  new->file = fopen(filename, "r");
  if( new->file == NULL ){
    error(-1,"Impossible de lire le fichier pav");
  }
  /* create fits header */
  new->nb_header_block = 1;
  
  /* find the number of keywords */
  new->nbKeys = 0;
  
  while(!end){
    char line[80];
    
    nread = fread(line,80, 1, new->file);
    if( nread!=1){
      error(-1," Problme de lecture du header du pave");
    }
    end = (strncmp(line,"END",3)==0);
    new->nbKeys++;
  } 
  new->header = (char*)(malloc(new->nbKeys*80*sizeof(char)));
  if( new->header == NULL ){
    error(-1,"Pas assez de mmoire pour crer le header d'un pav");
  }
  new->nb_header_block = ceil((double)new->nbKeys/36.0);
   
  rewind(new->file);
  nread = fread(new->header,80*new->nbKeys, 1, new->file);
  if( nread!=1){
      error(-1," Problme de lecture du header du pave");
  }
  fclose(new->file);

  if(pave_keyword_get(new,"NAXIS1",value)){
    new->sizeX = atoi(value);
  }
  else {
    error(-1,"NAXIS1 introuvable dans le header du pav");
  }
  if(pave_keyword_get(new,"NAXIS2",value)){
    new->sizeY = atoi(value);
  }
  else {
    error(-1,"NAXIS2 introuvable dans le header du pav");
  }
  if(pave_keyword_get(new,"NXPAVMAX",value)){
    new->nxpavmax = atoi(value);
  }
  else {
    error(-1,"NXPAVMAX introuvable dans le header du pav");
  }
  if(pave_keyword_get(new,"NYPAVMAX",value)){
    new->nypavmax = atoi(value);
  }
  else {
    error(-1,"NYPAVMAX introuvable dans le header du pav");
  }
  if(pave_keyword_get(new,"CDELT1",value)){
    new->cdelt1 = atof(value);
  }
  else {
    error(-1,"CDELT1 introuvable dans le header du pav");
  }
  if(pave_keyword_get(new,"CDELT2",value)){
    new->cdelt2 = atof(value);
  }
  else {
    error(-1,"CDELT2 introuvable dans le header du pav");
  }
  if(pave_keyword_get(new,"YOFFSET",value)){
    new->yoffset = atoi(value);
  }
  else {
    error(-1,"YOFFSET introuvable dans le header du pav");
  }
  if(pave_keyword_get(new,"XOFFSET",value)){
    new->xoffset = atoi(value);
  }
  else {
    error(-1,"XOFFSET introuvable dans le header du pav");
  }
  return new;
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* pave_free
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void pave_free(pave* pav){
  if(pav){
    free(pav->path);
    free(pav->format);
    if( pav->header ){
      free(pav->header);
    }
    if( pav->pixels ){
      free(pav->pixels);
    }
    free(pav);
  }
}
/*
----------------------------------------------------------------------------------------
*/
/*
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* pave_image
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
void pave_image(pave* pav,int numPav){
  char filename[256];
  int nread;
  
  /* construct the filename */
  sprintf(filename,pav->format,pav->path,numPav);
  /* open the file in read */
  pav->file = fopen(filename, "r");
  if( pav->file == NULL ){
    error(-1,"Impossible de lire le fichier pav");
  }
  /* allocate memory */
  if( !pav->pixels ){
    pav->pixels = (PIXTYPE*)(malloc(pav->sizeX*pav->sizeY*sizeof(PIXTYPE)));
  }
  /* go to the pixels */
  fseek(pav->file,pav->nb_header_block*2880,SEEK_SET);
  /* read the pixels */
  nread = fread(pav->pixels, pav->sizeX * pav->sizeY *sizeof(PIXTYPE),1,pav->file);

  if ( nread != 1 ){
    error(-1," Impossible de lire le pave");
  }
  fclose(pav->file);
}

/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* pave_keyword_get
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int pave_keyword_get(pave* pav,char* keyword,char* value){
  int i;
  
  for(i=0;i<pav->nbKeys;i++){
    char* line;
    
    line = &pav->header[80*i];
    if( keyword_get(keyword,value,line) )return 1;
  }
  return 0;
}
/*
----------------------------------------------------------------------------------------
*/
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* pave_keyword_set
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/
int pave_keyword_set(pave* pav,char* keyword,char* value){
  int i,j,k;
  char oldvalue[30];
  

  for(i=0;i<pav->nbKeys;i++){
    char* line;
    
    line = &pav->header[80*i];
    if( keyword_get(keyword,oldvalue,line) ){
      int size;
      
      size = strlen(value);
      
      if(size > 70){
        size = 70;
      }
      for (j = 0; j < size; j++){
        if(line[10+ j]='/'){
	  for(k=j;k<size;k++){
	    line[10+k] = ' ';
	  }
	}
	line[10+j] = value[j];
      }
      return 1;
    }
  }
  return 0;
}
/*
----------------------------------------------------------------------------------------
*/
