#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>
#include <values.h>  /* M_PI defintion */
#include <fits.h>

#define MODE_RENAME 1
#define MODE_UPDATE 2
#define MREPLACEKEY_SYNTAX \
"mreplacekey replace a key in a fits file\n\
               -mode <mode rename or update>\n\
               -file <fits file to update>\n\
               -key <keyword to update>\n\
	       -value <[update => new value + comment or rename => new key>\n"

int main(int argc,char* argv[]){
  char* imagename = NULL;
  char* keyword = NULL;
  int mode = 0;
  char* value = NULL;
  int i = 0,nbread = 0;
  fits* result = NULL;
  FILE* file = NULL;

  if(argc < 8){
    printf("argc %d\n%s",MREPLACEKEY_SYNTAX);
    exit(-1);
  }
  for( i=1;i<argc;i++ ){
    if( strcmp("-mode",argv[i])==0 ){
      i++;
      if(strcmp(argv[i],"rename")==0) mode = MODE_RENAME;
      if(strcmp(argv[i],"update")==0) mode = MODE_UPDATE;
      if(mode == 0) {
        printf("mode %s\n%s",argv[++i],MREPLACEKEY_SYNTAX);
	exit(-1);
      }
    }
    else if( strcmp("-file",argv[i])==0 ){
      imagename = strdup(argv[++i]);  
    }
    else if(strcmp("-key",argv[i])==0 ){
      keyword = strdup(argv[++i]);
    }
    else if( strcmp("-value",argv[i])==0 ){
      value = strdup(argv[++i]);
    }
    else{
      printf("argv %s\n%s",argv[i],MREPLACEKEY_SYNTAX);
      exit(-1);
    }
  }
  
  /* check parameters */
  if (imagename == NULL){
    printf("ERROR invalid -image parameter\n");
    exit(-1);
  }
  
  if ( (keyword == NULL) || (strlen(keyword) > 8) ){
    printf("ERROR invalid -keyword parameter\n");
    exit(-1);
  }

  if (value == NULL){
    printf("ERROR invalid -value parameter\n");
    exit(-1);
  }

  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("MREPLACEKEY\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("Image file = %s\n", imagename);
  printf("Keyword  = %s\n", keyword);
  printf("Value  = %s\n", value);

  result = fits_new(imagename);

  if(mode == MODE_RENAME) fits_keyword_rename(result,keyword,value);
  if(mode == MODE_UPDATE) fits_keyword_set(result,keyword,value);
  /* write the result file */
  file = fopen(imagename,"r+");
  if( file == NULL ){
    mama_error(-1,"Can't update file");
  }

/* write the header */
  nbread = fwrite(result->header, 2880 * result->nb_header_block,1,file);
  if (nbread != 1) {
    mama_error(-1,"Can't write the header in the file");
  }

  fclose(file);

  /* free parameters */
  free(imagename);
  free(keyword);
  free(value);

  return 0;
}



