#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>
#include <fits.h>
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* SYNTAX definitions
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

#define        MSPLIT_SYNTAX \
"msplit     version " PACKAGE_VERSION "\n"\
"            extract image borders\n\
             -file        <image to cut>\n\
             -resdir      <res dir>\n\
             -nx          <number pav in x>\n\
             -ny          <number pav in y>\n"

#define XPAVSIZE 1024  /* in pixels */
#define YPAVSIZE 1024

int main(int argc,char* argv[]){
  char* filename = NULL;
  char* dirname = NULL;
  char* root = NULL;
  char name[1024];
  fits* source = NULL;
  int nx=33, ny=33;
  int i, j;
  char* buff16 = NULL;

  if(argc < 8){
    printf("%s",MSPLIT_SYNTAX);
    exit(-1);
  }
  for( i=1;i<argc;i++ ){
    if( strcmp("-file",argv[i])==0 ){
      filename = strdup(argv[++i]);
      filename[strlen(argv[i])]='\0';
    }
    else if ( strcmp("-resdir",argv[i])==0 ){
      dirname = strdup(argv[++i]);
      dirname[strlen(argv[i])]='\0';
    }
    else if ( strcmp("-nx",argv[i])==0 ){
      nx = atoi(argv[++i]);
    }
    else if ( strcmp("-ny",argv[i])==0 ){
      ny = atoi(argv[++i]);
    }
    else {
      printf("%s",MSPLIT_SYNTAX);
      exit(-1);
   }
 }
  
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("MSPLIT\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("Source file       = %s\n", filename);
  printf("Result Directory  = %s\n", dirname);
  printf("NX %d NY %d\n",nx,ny);

  source = fits_new(filename);
  fits_seek_pixels(source);
  
  root = strdup(strrchr(filename,'/'));
  root = strtok(root,".");

  buff16 = (char*)(malloc(XPAVSIZE*2*sizeof(char)));

  for(j=0; j< nx; j++){
    for(i=1; i<= ny; i++){
      int ideb,nb,k;
      FILE* file = NULL;
      sprintf(name,"%s%s%04d.fits",dirname,root,i+j*33);
      printf("name %s\n",name);
      fits_seek_pixels(source);
      ideb = (i-1)*source->naxis1*1024+j*1024;

      file = fopen(name,"w");
      if( file == NULL ){
        mama_error(-1,"Can't write in result file");
      }
      for(k=0; k< 1024; k++){
        fits_seek_pixels(source);
        fseek(source->file,(ideb+k*source->naxis1)*2, SEEK_CUR);
        nb = fread( buff16, XPAVSIZE*2, sizeof(char), source->file);
        nb = fwrite(buff16, XPAVSIZE*2,sizeof(char),file);
      }
      fclose(file);
    }
  }

  free(buff16);
  fits_free(source);
  free(filename);
  free(dirname);
  return 0;

}



