#include <utils.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <math.h>
#include <fits.h>
/*
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* SYNTAX definitions
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

#define        WEIGHTMAP_SYNTAX \
"weightmap     version " PACKAGE_VERSION "\n"\
"            create weight map for FITS file\n\
             -file       <image to weight>\n\
             -poly       <poly file don't close it>\n\
             -out        <result file>\n"

/* 
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Prototypes
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
*/
static void read_poly_file( char *name, int **x, int**y, int* size);
static void tab_get_min_max( int *tab, int size, int *min, int *max);
static int is_in_poly( int* xp, int* yp, int size, int x, int y);
/* 
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Prototypes
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
*/
static void read_poly_file(char *name, int **x, int**y, int* size)
{
  FILE* file = NULL;
  char record[90];
  int i;
  int file_size = 0;
  int *tabx = NULL;
  int *taby = NULL;

  MAMA_OPEN_FILE( file, name, "r");

while (fgets (record, 90, file) != NULL){
    file_size++;
  }
  tabx = calloc( file_size, sizeof(int));
  taby = calloc( file_size, sizeof(int));

  rewind( file);

  for( i=0; i< file_size; i++){
    fscanf( file, "%d %d", &tabx[i], &taby[i]);
  }
  *x = tabx;
  *y = taby;
  *size = file_size;
  fclose(file);
}



static void tab_get_min_max( int* tab, int size, int* min, int* max)
{
  int i;

  MAMA_NOT_NULL( tab);
  MAMA_NOT_NULL( min);
  MAMA_NOT_NULL( max);

  *min = tab[0];
  *max = tab[0];
  for( i=1; i<size; i++){
    if ( tab[i] > *max){
      *max = tab[i];
    }
    else if (tab[i] < *min){
      *min = tab[i];
    }
  }
}

static int is_in_poly( int* xp, int* yp, int size, int x, int y)
{
  int i ,j ,c = 0;

  for( i=0, j = size-1; i<size; j=i++){
    if ((((yp[i] <= y ) && ( y < yp[j])) ||
         (( yp[j] <= y ) && ( y < yp[i]))) &&
	( x < (xp[j]-xp[i]) * (y-yp[i])/(yp[j]-yp[i])+xp[i])){
	  c = !c;
    }
  }
  return c;
}


int main(int argc,char* argv[]){
  char *image_file_name = NULL;
  char *poly_file_name = NULL;
  char *outname = NULL;
  fits* f = NULL, *wmap = NULL;
  int *xp = NULL;
  int *yp = NULL;
  int size = 0, ymin = 0, ymax = 0, xmin = 0, xmax = 0;
  char value[22];
  char* buff8 = NULL;
  FILE* file = NULL;
  int nb,i,j;

  if(argc < 5){
    printf("not enough argument\n%s",WEIGHTMAP_SYNTAX);
    exit(-1);
  }
  for( i=1;i<argc;i++ ){
    if( strcmp("-file",argv[i])==0 ){
      image_file_name = strdup(argv[++i]);
    }
    else if ( strcmp("-out",argv[i])==0 ){
      outname = strdup(argv[++i]);
    }
    else if ( strcmp("-poly",argv[i])==0 ){
      poly_file_name = strdup(argv[++i]);
    }
    else {
      printf("invalid option %s\n%s",argv[i],WEIGHTMAP_SYNTAX);
      exit(-1);
   }
 }

  if ( image_file_name == NULL){
    printf("invalid image_file_name\n");
    exit(-1);
  }
  if ( outname == NULL){
    printf("invalid outname\n");
    exit(-1);
  }
  if ( poly_file_name == NULL){
    printf("invalid outname\n");
    exit(-1);
  }

  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("WEIGHTMAP\n");
  printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
  printf("Source file  = %s\n", image_file_name);
  printf("Poly file = %s\n", poly_file_name);
  printf("Out file  = %s\n", outname);

  read_poly_file( poly_file_name, &xp, &yp ,&size);
  tab_get_min_max( xp, size, &xmin, &xmax);
  tab_get_min_max( yp, size, &ymin, &ymax);

  printf("xmin %d ymin %d xmax %d ymax %d\n",xmin,ymin,xmax,ymax);
  f = fits_new( image_file_name);

  /* verify the input parameters */
  if(f->bitpix != 16){
    mama_error(-1,"Can't convert this file bitpix != 16");
  }

  wmap = fits_template_new(f, outname);
  fits_set_bitpix(wmap,8);

  sprintf(value,"%20f", 0.0);
  fits_keyword_set(wmap,"BZERO",value);
  sprintf(value,"%20f", 1.0);
  fits_keyword_set(wmap,"BSCALE",value);

  /* write the result file */
  file = fopen(outname, "w");
  if( file == NULL ){
    mama_error(-1,"Can't write in result file");
  }
  nb = fwrite(wmap->header, 2880 * wmap->nb_header_block,1,file);

  if (nb != 1) {
    mama_error(-1,"Can't write the header");
  }

  /* write the pixel */
  buff8 = (char*)(malloc(wmap->naxis1*sizeof(char)));

  for( j=0;j< ymin;j++ ){
    for( i=0;i< wmap->naxis1;i++ ){
      buff8[i]=(char)(1);
    }
    nb = fwrite( buff8, wmap->naxis1, sizeof(char), file);
  }

  for( j=ymin;j< ymax;j++ ){
    for( i=0;i< wmap->naxis1;i++ ){
      if (i>= xmin && i <= xmax){
        int eval = is_in_poly( xp, yp, size, i, j);
        if ( eval == 1){
	  printf("Inside \n");
          buff8[i]=(char)(0);
	}
	else{
	  buff8[i]=(char)(1);
	}
	printf( "Y %d X %d is in %d\n", j, i, eval);
      }
      else{
        buff8[i]=(char)(1);
      }
    }
    nb = fwrite( buff8, wmap->naxis1, sizeof(char), file);
  }

  for( j=ymax;j< wmap->naxis2;j++ ){
    for( i=0;i< wmap->naxis1;i++ ){
      buff8[i]=(char)(1);
    }
    nb = fwrite( buff8, wmap->naxis1, sizeof(char), file);
  }
  
  /* complete with zero */
  fits_complete_with_zero( wmap,file);
  free(buff8); 
  fclose(file);
  fits_free(f);
  fits_free(wmap);
  free(image_file_name);
  free(outname);
  free(poly_file_name);
  free( xp);
  free( yp);
  return 0;

}



